#include "attributedatatreeview.h"
#include "modelview/itemmodel.h"
#include "utils/helperapplication.h"
#include "selection/selectiondialog.h"
#include "settings/settings.h"
#include <QHeaderView>
#include <QVariant>
#include <QContextMenuEvent>
#include <QMenu>
#include <QPainter>
#include <QPaintEvent>
#include <QtDebug>
#include <tutil.h>
#include <macros.h>
#include <elettracolors.h>
#include "treeviewfilter.h"
#include "invaliditemmanager.h"
#include "snapinfotreeviewsorter.h"
#include "uihelper.h"

AttributeDataTreeView::AttributeDataTreeView(QWidget *parent) : QTreeView(parent)
{
    m_currentContextId = m_currentSnapId = -1;
}

void AttributeDataTreeView::init()
{
    qDebug() << __FUNCTION__;
    setRootIsDecorated(false);

#if QT_VERSION >= 0x050000
    header()->setSectionResizeMode(QHeaderView::Interactive);
#else
    header()->setResizeMode(QHeaderView::Interactive);
#endif
    header()->setToolTip("Click to switch sorting mode");
    setSortingEnabled(true);
    connect(header(), SIGNAL(sectionClicked(int)), this, SLOT(switchSortMode(int)));

    ItemModel *itemModel = new ItemModel(this, getHeaderData());

    setModel(itemModel);

    stateChanged(Init); /* takes care of initializing SelectionMode to NoSelection */

    setSelectionBehavior(QAbstractItemView::SelectRows);
    setDragEnabled(false);
    setDragDropMode(QAbstractItemView::NoDragDrop);

    setStatusTip("Snap Info View: click to select/deselect for restore. Right click "
                 "for context menu.");
    /* selected items green, not blue as the default, indicating that they will be restored */
    QPalette p = palette();
    p.setColor(QPalette::Highlight, KGREEN.lighter(120));
    p.setColor(QPalette::HighlightedText, Qt::black);
    setPalette(p);

    m_invalidItemManager = new InvalidItemManager(this);
    m_currentLastExpectedType = AttInfo::Invalid;

    connect(new TreeViewFilter(this), SIGNAL(regexpValid(bool)), this, SIGNAL(filterRegexpValid(bool)));
}

QStringList AttributeDataTreeView::getAttributeList()
{
    QStringList atts;
    ItemModel *itemModel = static_cast<ItemModel *>(model());
    for(int i = 0; i < itemModel->rowCount(); i++)
        atts << itemModel->index(i, getColumnIndex(ColumnAttr)).data().toString();
    qDebug() << __FUNCTION__ << itemModel->rowCount();
    return atts;
}

AttributeDataTreeView::Mode AttributeDataTreeView::getMode() const
{
   return Snap;
}

QList<QVariant> AttributeDataTreeView::getHeaderData()
{
    return QList<QVariant>() << "Attribute" << "Set Point" << "Read" << "Type" << "Format" << "Writable";
}

void AttributeDataTreeView::clear()
{
    model()->removeRows(0, model()->rowCount());
}

void AttributeDataTreeView::update(const QList<AttInfo>& attInfoList)
{
    m_currentAttInfoList = attInfoList;
    /* clear view */
    model()->removeRows(0, model()->rowCount());
    ItemModel *itemModel = static_cast<ItemModel *>(model());

    itemModel->blockSignals(true);
    selectionModel()->blockSignals(true);

    int row = -1;

    /* clear att id / row hash */
    m_idRowHash.clear();

    foreach(AttInfo ai, attInfoList)
    {
        row = model()->rowCount(); /* a new row is inserted below */
        itemModel->insertRows(model()->rowCount(), 1); /* one row is added here */
        /* the hash will be used in the other update method to fastly retrieve the row
         * where the attribute with id equal to ai.id is stored in the model.
         * This hash assumes that the rows are never sorted.
         */
        m_idRowHash.insert(ai.id, row);

        /* final false parameter not to emit dataChanged at every setData */
        itemModel->setData(model()->index(row, getColumnIndex(ColumnAttr)), ai.name, Qt::DisplayRole, false);
        itemModel->setData(model()->index(row, getColumnIndex(ColumnAttr)), ai.id, Qt::UserRole, false);

        /* empty fields in this phase */
        itemModel->setData(model()->index(row, getColumnIndex(ColumnSetPoint)), "-", Qt::DisplayRole, false);
        itemModel->setData(model()->index(row, getColumnIndex(ColumnRead)), "-", Qt::DisplayRole, false);

        /* data writable type format */
        itemModel->setData(model()->index(row, getColumnIndex(ColumnType)), ai.attrDataType(), Qt::DisplayRole, false);
        itemModel->setData(model()->index(row, getColumnIndex(ColumnType)), ai.dataType, Qt::UserRole, false);

        itemModel->setData(model()->index(row, getColumnIndex(ColumnFormat)), ai.dataFormat(), Qt::DisplayRole, false);
        itemModel->setData(model()->index(row, getColumnIndex(ColumnFormat)), ai.format, Qt::UserRole, false);

        itemModel->setData(model()->index(row, getColumnIndex(ColumnWritable)), ai.writeType(), Qt::DisplayRole, false);
        itemModel->setData(model()->index(row, getColumnIndex(ColumnWritable)), ai.writable, Qt::UserRole, false);
    }
    /* trigger an update from the top left to the bottom right index */
    itemModel->update(itemModel->index(0, 0), itemModel->index(row, 3));
    header()->resizeSections(QHeaderView::ResizeToContents);

    itemModel->blockSignals(false);
    selectionModel()->blockSignals(false);

    stateChanged(InfoLoaded);

    emit attributeInfoDisplayed();
}

int AttributeDataTreeView::getColumnIndex(Columns c) const
{
    return c;
}

int AttributeDataTreeView::getRestoreColumnIndex() const
{
    return getColumnIndex(ColumnSetPoint);
}

void AttributeDataTreeView::update(const QList<AttData>& values, AttInfo::Type type)
{
    ItemModel *itemModel = static_cast<ItemModel *>(model());    
    itemModel->blockSignals(true);
    selectionModel()->blockSignals(true);

    foreach(AttData av, values)
    {
        if(av.idSnap != m_currentSnapId)
        {
            pinfo("SnapInfoTreeView::update(): snap id not matching (%d/%d)",
                  av.idSnap, m_currentSnapId);
            return;
        }
        int row = m_idRowHash.value(av.id);
        QString truncValue = av.value;
        QString truncSetPoint = av.setpoint;
        QString dateTime = av.dateTime.toString();
        truncValue.truncate(30);
        truncSetPoint.truncate(30);
        /* show the truncated version into the Display Role for performance issues */
        itemModel->setData(model()->index(row, getColumnIndex(ColumnRead)), truncValue, Qt::DisplayRole, false);

        /*
         *  IMPORTANT NOTE: save the full data into the user role.
         */
        itemModel->setData(model()->index(row,  getColumnIndex(ColumnRead)), av.value, Qt::UserRole, false);

        itemModel->setData(model()->index(row, getColumnIndex(ColumnRead)), "<p>" + av.value + "</p>",
                           Qt::ToolTipRole, false);

        if(av.type == AttInfo::Im2 || av.type == AttInfo::scNum2 || av.type == AttInfo::sp2
                || av.type == AttInfo::scStr2)
        {
            itemModel->setData(model()->index(row, getColumnIndex(ColumnSetPoint)),
                               truncSetPoint, Qt::DisplayRole, false);


            /*
             *  IMPORTANT NOTE: save the full data into the user role.
             */
            itemModel->setData(model()->index(row, getColumnIndex(ColumnSetPoint)), av.setpoint, Qt::UserRole, true);
            itemModel->setData(model()->index(row, getColumnIndex(ColumnSetPoint)),  "<p>" + av.setpoint + "</p>",
                               Qt::ToolTipRole, false);
        }
    }

    itemModel->blockSignals(false);
    selectionModel()->blockSignals(false);

    stateChanged(ValuesLoaded);
    /* invalid or read only items in the view ? */
    if(type == m_currentLastExpectedType)
    {
        bool invalidIts = m_invalidItemManager->findInvalidItems();
        if(invalidIts)
            emit invalidItems(m_invalidItemManager->invalidWriteModeItems(), m_invalidItemManager->nullItems());
        emit allItemsValid(!invalidIts);
        emit attributeValuesDisplayed();
    }
    dataChanged(model()->index(0, model()->columnCount() -1),
                model()->index(model()->rowCount() -1, model()->columnCount() - 1));
}

void AttributeDataTreeView::stateChanged(State st)
{
    m_state = st;
    switch(st)
    {
    case Init:
    case InfoLoaded:
        setSelectionMode(QAbstractItemView::NoSelection);
        break;
    case ValuesLoaded:
        setSelectionMode(QAbstractItemView::MultiSelection);
        break;
    default:
        break;
    }
}

void AttributeDataTreeView::prepareForAttributeValuesQuery()
{
    m_sp2val.clear();
    m_sp1val.clear();
    m_im2val.clear();
    m_im1val.clear();
    m_sc2valStr.clear();
    m_sc1valStr.clear();
    m_sc2val.clear();
    m_sc1val.clear();

    //    printf("\e[0;33mprepareForAttributeValuesQuery(): there are %d items in the current list\e[0m\n",
    //           m_currentAttInfoList.size());

    int attId;
    foreach(AttInfo ai, m_currentAttInfoList)
    {
        AttInfo::Type itype = AttInfo::getType(ai.dataType, ai.format, ai.writable);
        attId = ai.id;

        switch(itype)
        {
        case AttInfo::scNum1:
            m_sc1val << attId;
            break;
        case AttInfo::scNum2:
            m_sc2val << attId;
            break;
        case AttInfo::scStr1:
            m_sc1valStr << attId;
            break;
        case AttInfo::scStr2:
            m_sc2valStr << attId;
            break;
        case AttInfo::Im1:
            m_im1val << attId;
            break;
        case AttInfo::Im2:
            m_im2val << attId;
            break;
        case AttInfo::sp1:
            m_sp1val << attId;
            break;
        case AttInfo::sp2:
            m_sp2val << attId;
            break;
        default:
            break;
        }
    }

    /* SnapInfoViewDbHelper gets the values in the same order as the Types appear in the AttInfo enum:
     * enum Type { Im1 = 1, Im2, scNum1, scNum2, scStr1, scStr2, sp1, sp2, Invalid };
     * So it's easy to find out what will be the last values we will obtain.
     */
    if(m_sp2val.size())
        m_currentLastExpectedType = AttInfo::sp2;
    else if(m_sp1val.size())
        m_currentLastExpectedType = AttInfo::sp1;
    else if(m_sc2valStr.size())
        m_currentLastExpectedType = AttInfo::scStr2;
    else if(m_sc1valStr.size())
        m_currentLastExpectedType = AttInfo::scStr1;
    else if(m_sc2val.size())
        m_currentLastExpectedType = AttInfo::scNum2;
    else if(m_sc1val.size())
        m_currentLastExpectedType = AttInfo::scNum1;
    else if(m_im2val.size())
        m_currentLastExpectedType = AttInfo::Im2;
    else if(m_im1val.size())
        m_currentLastExpectedType = AttInfo::Im1;

    ItemModel *itemModel = static_cast<ItemModel *>(model());
    for(int i = 0; i < itemModel->rowCount(QModelIndex()); i++)
    {
        int wri = itemModel->data(itemModel->index(i, ColumnWritable), Qt::UserRole).toInt();
        /* prepare Set Point and Read columns with NULL values */
        itemModel->setData(model()->index(i, ColumnRead), "NULL", Qt::DisplayRole, true);
        itemModel->setData(model()->index(i, ColumnRead), "NULL", Qt::ToolTipRole, true);

        Tango::AttrWriteType writable = static_cast<Tango::AttrWriteType>(wri);
        if(writable == Tango::READ_WRITE || writable ==  Tango::READ_WITH_WRITE)
        {
            itemModel->setData(model()->index(i, ColumnSetPoint), "NULL", Qt::DisplayRole, true);
            itemModel->setData(model()->index(i, ColumnSetPoint), "NULL", Qt::ToolTipRole, true);
        }
        else
        {
            itemModel->setData(model()->index(i, ColumnSetPoint), "-", Qt::DisplayRole, true);
            itemModel->setData(model()->index(i, ColumnSetPoint), "-", Qt::ToolTipRole, true);
        }
    }
}

QList<int> AttributeDataTreeView::im1ValAttributes() const
{
    return m_im1val;
}

QList<int> AttributeDataTreeView::im2ValAttributes() const
{
    return m_im2val;
}

QList<int> AttributeDataTreeView::sc1ValAttributes() const
{
    return m_sc1val;
}

QList<int> AttributeDataTreeView::sc2ValAttributes() const
{
    return m_sc2val;
}

QList<int> AttributeDataTreeView::sp1ValAttributes() const
{
    return m_sp1val;
}

QList<int> AttributeDataTreeView::sp2ValAttributes() const
{
    return m_sp2val;
}

QList<int> AttributeDataTreeView::sc1ValStringAttributes() const
{
    return m_sc1valStr;
}

QList<int> AttributeDataTreeView::sc2ValStringAttributes() const
{
    return m_sc2valStr;
}

void AttributeDataTreeView::contextMenuEvent(QContextMenuEvent *e)
{
    QMenu *menu = NULL;
    bool indexValid = indexAt(viewport()->mapFromGlobal(e->globalPos())).isValid();
    if(model()->rowCount() > 0)
    {
        menu = new QMenu(this);
        menu->setObjectName("contextMenu");
    }
    else
        return;

    if(m_state == ValuesLoaded)
    {
        menu->addAction("Selection...", this, SIGNAL(selectionRequest()));
        menu->addSeparator();
        menu->addAction("Select All", this, SLOT(selectAll()));
        menu->addAction("Select None", this, SLOT(clearSelection()));
    }
    if(indexValid)
    {
        menu->addSeparator();
        menu->addAction("Helper Application", this, SLOT(launchHelperApplication()));
    }
    /* filter request action */
    if(menu->actions().count() > 0)
        menu->addSeparator();
    menu->addAction("Filter view...", this, SIGNAL(showFilterRequest()));

    if(m_orderedSectionsList.size() > 0)
    {
        menu->addSeparator();
        QAction *sortBySectionAction = new QAction("Sort by section", menu);
        menu->addAction(sortBySectionAction);
        sortBySectionAction->setCheckable(true);
        Settings settings;
        sortBySectionAction->setChecked(settings.sectionSortingForContext(m_currentContextId));
        connect(sortBySectionAction, SIGNAL(toggled(bool)), this, SLOT(sortBySection(bool)));
    }

    if(menu)
    {
        menu->exec(QCursor::pos());
        delete menu;
    }
}

void AttributeDataTreeView::launchHelperApplication()
{
    QMenu *menu = findChild<QMenu *>("contextMenu");
    if(menu)
    {
        QPoint pt = viewport()->mapFromGlobal( menu->pos());
        QModelIndex idx = indexAt(pt);
        if(idx.isValid())
        {
            QModelIndex srcIdx = model()->index(idx.row(), 0);
            HelperApplication helperApplication;
            QString source = srcIdx.data(Qt::DisplayRole).toString();
            if(!helperApplication.launch(source))
                emit error("Snap Info View",
                           QString("No helper application available for \"%1\"").arg(source));
        }
    }
}

void AttributeDataTreeView::selectAll()
{
    QItemSelection itemSelection(model()->index(0, 0), model()->index(model()->rowCount() -1,
                                                                      model()->columnCount() -1));
    selectionModel()->select(itemSelection, QItemSelectionModel::Select);
}

void AttributeDataTreeView::selectByDevice(const QStringList &devList)
{
    qDebug() << __FUNCTION__ << devList;
    selectionModel()->clear();
    QModelIndexList indexList;
    for(int i = 0; i < model()->rowCount(); i++)
    {
        QString source = model()->data(model()->index(i, ColumnAttr), Qt::DisplayRole).toString();
        if(devList.contains(m_devFromSource(source)))
            indexList << model()->index(i, 0);
    }
    for(int i = 0; i < indexList.size(); i++)
    {
        QItemSelection itemSelection(model()->index(indexList.at(i).row(), 0),
                                     model()->index(indexList.at(i).row(),
                                                    model()->columnCount() - 1));
        selectionModel()->select(itemSelection, QItemSelectionModel::Select);
    }
}

QString AttributeDataTreeView::m_devFromSource(const QString &src) const
{
    QString ret;
    QStringList sl;
    if(src.count("/") == 3)
        sl = src.split("/", QString::SkipEmptyParts);
    if(sl.size() == 4)
        ret = sl.at(0) + "/" + sl.at(1) + "/" + sl.at(2);
    return ret;
}

TreeViewFilter *AttributeDataTreeView::treeViewFilter() const
{
    return findChild<TreeViewFilter *>();
}

void AttributeDataTreeView::setFilterMode(int mode)
{
    findChild<TreeViewFilter *>()->setFilterMode(static_cast<TreeViewFilter::FilterMode>(mode));
}

void AttributeDataTreeView::filter(const QString& text)
{
    findChild<TreeViewFilter *>()->filter(text, 0);
}

void AttributeDataTreeView::setSectionsList(const QStringList& orderedSections)
{
    m_orderedSectionsList = orderedSections;
}

void AttributeDataTreeView::switchSortMode(int columnClicked)
{
    if(columnClicked != 0)
        return;
}

void AttributeDataTreeView::sortBySection(bool bySection)
{
    /* store on app settings the last desired sorting mode */
    Settings s;
    qDebug() << "sortBySection " << m_currentContextId << bySection;
    s.setSectionSortingForContext(m_currentContextId, bySection);
    this->selectionModel()->blockSignals(true); /* avoid selection changed signal */
    SnapInfoTreeViewSorter sorter(this, bySection, m_orderedSectionsList);
    this->selectionModel()->blockSignals(false);
    qDebug() << "emetto snapIngortree sorted" << m_currentSnapId;
    emit snapInfoTreeViewSorted(m_currentSnapId);
    /* restore filter, if visible */
    this->filter(findChild<TreeViewFilter *>()->filterText());
}

