#include "snapinsertquery.h"
#include <macros.h>
#include <QSqlError>
#include <QSqlQuery>

SnapInsertQuery::~SnapInsertQuery()
{
    pdelete("deleting snap insert query");
}

SnapInsertQuery::SnapInsertQuery(QObject *parent) :
    DbQueryInterface(parent)
{
    m_snapshotId = -1;
}

void SnapInsertQuery::reset()
{
    m_data.clear();
}

void SnapInsertQuery::setData(AttInfo::Type t, const QList<AttData> &v)
{
    m_data.insert(t, v);
}

void SnapInsertQuery::notify(const QSqlQuery& q, const int eventId)
{
    QSqlError qSqlError = q.lastError();
    if(qSqlError.type() != QSqlError::NoError)
        m_errorMessage = qSqlError.text();
    else
        m_errorMessage.clear();

    if(!m_errorMessage.isEmpty())
        emit queryExecuted(q.lastQuery(), false, qSqlError.text());
    else
        emit queryExecuted(q.lastQuery(), true, QString());

    printf("\e[1;35mSnapInsertQuery::notify() leaving\e[0m\n");
}

void SnapInsertQuery::execute(int id)
{
    /* works with data types different from Tango::IMAGE.
     */
    m_errorMessage.clear(); /* clear error message field */

    AttInfo::Type type = static_cast<AttInfo::Type> (id);
    if(type == AttInfo::Im1 || type == AttInfo::Im2)
        m_errorMessage = "Image type unsupported";

    if(!m_errorMessage.isEmpty()) /* Image data type */
        emit queryExecuted("SnapInsertQuery::execute()", false, m_errorMessage);
    else
        DbQueryInterface::execute(id);
}

QString SnapInsertQuery::query(int id) const
{
    QString q;
    QString vls;
    AttInfo::Type type = static_cast<AttInfo::Type> (id);
    if(m_data.contains(type) && m_snapshotId > 0)
    {
        QList<AttData> values = m_data.value(type);
        if(values.size() > 0)
        {
            vls.clear();

            switch(type)
            {
            case  AttInfo::scNum1:

                foreach(AttData av, values)
                {
                    if(av.valid())
                        vls += QString("(%1,%2,%3),").arg(m_snapshotId).arg(av.id).arg(av.value);
                }
                if(vls.endsWith(','))
                    vls.remove(vls.size() - 1, 1); /* remove last comma */
                if(!vls.isEmpty())
                    q = QString("INSERT INTO t_sc_num_1val VALUES%1").arg(vls);
                break;

            case  AttInfo::scStr1:

                /* NOTE: string between ' in query */
                foreach(AttData av, values)
                {
                    if(av.valid())
                        vls += QString("(%1,%2,'%3'),").arg(m_snapshotId).arg(av.id).arg(av.value);
                }
                if(vls.endsWith(','))
                    vls.remove(vls.size() - 1, 1); /* remove last comma */
                /* t_sc_str_1val table */
                if(!vls.isEmpty())
                    q = QString("INSERT INTO t_sc_str_1val VALUES%1").arg(vls);
                break;

            case  AttInfo::scNum2:

                foreach(AttData av, values)
                {
                    if(av.valid())
                       vls += QString("(%1,%2,%3,%4),").arg(m_snapshotId).arg(av.id)
                               .arg(av.value).arg(av.setpoint);
                }
                if(vls.endsWith(','))
                    vls.remove(vls.size() - 1, 1); /* remove last comma */
                if(!vls.isEmpty())
                    q = QString("INSERT INTO t_sc_num_2val VALUES%1").arg(vls);
                break;

            case  AttInfo::scStr2:

                foreach(AttData av, values)
                {
                    if(av.valid())
                        vls += QString("(%1,%2,'%3','%4'),").arg(m_snapshotId).arg(av.id)
                            .arg(av.value).arg(av.setpoint);
                }
                if(vls.endsWith(','))
                    vls.remove(vls.size() - 1, 1);
                /* t_sc_str_2val table */
                if(!vls.isEmpty())
                    q = QString("INSERT INTO t_sc_str_2val VALUES%1").arg(vls);
                break;

            case  AttInfo::sp1:
                /* spectra are saved in a list of comma separated elements.
                 * So we insert the value between '
                 * dimX is the number of elements read by the collector at the moment
                 * the reading was performed (see reader.cpp)
                 *
                 * describe t_sp_1val: id_snap,id_att,dim_x,value
                 */

                foreach(AttData av, values)
                {
                    if(av.valid())
                        vls += QString("(%1,%2,%3,'%4'),").arg(m_snapshotId).arg(av.id).
                            arg(av.dimX).arg(av.value);
                }
                if(vls.endsWith(','))
                    vls.remove(vls.size() - 1, 1);
                if(!vls.isEmpty())
                    q = QString("INSERT INTO t_sp_1val VALUES%1").arg(vls);
                break;

            case  AttInfo::sp2:

                foreach(AttData av, values)
                {
                    if(av.valid())
                       vls += QString("(%1,%2,%3,'%4','%5'),").arg(m_snapshotId).arg(av.id).
                            arg(av.dimX).arg(av.value).arg(av.setpoint);
                }
                if(vls.endsWith(','))
                    vls.remove(vls.size() - 1, 1);
                if(!vls.isEmpty())
                    q = QString("INSERT INTO t_sp_2val VALUES%1").arg(vls);
                break;

            /* unsupported or unrecognized types */
            case  AttInfo::Im1:
            case  AttInfo::Im2:
                perr("SnapInsertQuery::query(): unsupported Tango::Image data type");
                break;

            default:
                perr("SnapInsertQuery::query(): unrecognized data type %d", type);
                break;
            }
        }
    }
    else
        perr("SnapInsertQuery::query() cannot form query (id %d, snapId %d)", id, m_snapshotId);

    return q;
}


