#include "diffdialog.h"
#include "difftreeview.h"
#include "diffitemreader.h"
#include <modelview/treeitem.h>
#include <QDoubleValidator>
#include "database/snapdb.h"
#include "treeviewfilter.h"
#include <macros.h>
#include <QtDebug>

DiffDialog::~DiffDialog()
{
    pdelete("deleting DiffDialog");
}

DiffDialog::DiffDialog(QWidget *parent, QList<int> snapList, int contextId) :
    QDialog(parent)
{
    ui.setupUi(this);
    m_snapDb = NULL;
    m_nextSnapIndex = 0;
    m_contextId = contextId;
    QList<QVariant> snapshots;
    foreach(int snapId, snapList)
        snapshots << snapId;
    ui.diffView->setSnapshotIds(snapshots);

    connect(ui.cbLiveView, SIGNAL(toggled(bool)), this, SLOT(liveViewEnabled(bool)));
    connect(ui.cbLiveView, SIGNAL(toggled(bool)), ui.diffView, SLOT(liveViewEnabled(bool)));
    connect(ui.diffView, SIGNAL(validSnapSelection(bool)), ui.pbSelectSnap, SLOT(setEnabled(bool)));
    connect(ui.diffView, SIGNAL(attInfoReady()), this, SLOT(getNextAttributeValues()));
    connect(ui.pbSelectSnap, SIGNAL(clicked()), this, SLOT(selectAndClose()));
    connect(ui.dsbDiffPercent, SIGNAL(valueChanged(double)), ui.diffView, SLOT(percentChanged(double)));
    connect(ui.cbHighlightIfDiff, SIGNAL(toggled(bool)),  ui.diffView, SLOT(setHighlightDiffEnabled(bool)));
    connect(ui.leNoHighlightThreshold, SIGNAL(textChanged(const QString&)), ui.diffView, SLOT(setNoHightlightThresh(const QString&)));
    connect(ui.cbDontHighlightIfLower, SIGNAL(toggled(bool)), ui.diffView, SLOT(setNoHighlightIfBelowThresholdEnabled(bool)));
    connect(ui.cbFilterOutNotHighlighted, SIGNAL(toggled(bool)), ui.diffView, SLOT(setFilterOutNotHighlighted(bool)));
    connect(ui.cbHighlightIfDiff, SIGNAL(toggled(bool)), ui.dsbDiffPercent, SLOT(setEnabled(bool)));
    /* double validator on leNoHighlightThreshold */
    ui.leNoHighlightThreshold->setValidator(new QDoubleValidator(this));
    ui.pbSelectSnap->setDisabled(true);
    ui.dsbDiffPercent->setDisabled(true);

    ui.cbHighlightIfDiff->setDisabled(true);
    ui.cbDontHighlightIfLower->setDisabled(true);
    ui.leNoHighlightThreshold->setDisabled(true);
    ui.cbFilterOutNotHighlighted->setDisabled(true);

    ui.diffView->setNoHighlightIfBelowThresholdEnabled(ui.cbDontHighlightIfLower->isChecked());
    ui.diffView->setNoHightlightThresh(ui.leNoHighlightThreshold->text());
    connect(ui.leFilter, SIGNAL(textChanged(QString)), this, SLOT(filterTextChanged(QString)));
    connect(ui.pbClearFilter, SIGNAL(clicked()), ui.leFilter, SLOT(clear()));
    connect(ui.cbFilterOptions, SIGNAL(currentIndexChanged(int)), this, SLOT(filterOptionsChanged(int)) );
    connect(ui.cbLiveView, SIGNAL(toggled(bool)), ui.cbHighlightIfDiff, SLOT(setEnabled(bool)));
    connect(ui.cbLiveView, SIGNAL(toggled(bool)), ui.dsbDiffPercent, SLOT(setEnabled(bool)));
    connect(ui.cbLiveView, SIGNAL(toggled(bool)), ui.cbDontHighlightIfLower, SLOT(setEnabled(bool)));
    connect(ui.cbLiveView, SIGNAL(toggled(bool)), ui.leNoHighlightThreshold, SLOT(setEnabled(bool)));
    connect(ui.cbLiveView, SIGNAL(toggled(bool)), ui.cbFilterOutNotHighlighted, SLOT(setEnabled(bool)));


    new TreeViewFilter(ui.diffView);
}

void DiffDialog::setSnapDb(SnapDb *sdb)
{
    m_snapDb = sdb;

    connect(m_snapDb, SIGNAL(attInfoListReady(QList<AttInfo>)), ui.diffView,
            SLOT(update(QList<AttInfo>)));

    connect(m_snapDb, SIGNAL(attValueListReady(QList<AttData>, AttInfo::Type)), ui.diffView,
            SLOT(update(QList<AttData>, AttInfo::Type)));

    /* must serialize attribute values retrieval... */
    connect(m_snapDb, SIGNAL(attValueListReady(QList<AttData>, AttInfo::Type)), this,
            SLOT(getNextAttributeValues()));

}

void DiffDialog::getDiff()
{
    if(!m_snapDb)
        return;

    /* for the desired contextId, getAttInfo is called once.
     */
    m_snapDb->getAttInfoList(m_contextId);
}

void DiffDialog::getNextAttributeValues()
{
    /* check for index size: the slot is invoked one time more than necessary.
     */
    if(m_nextSnapIndex < ui.diffView->snapshotIds().size())
    {

        /* see saverestore.cpp code in method getAttributeValuesForSnap()
     *
     */
        printf("\e[1;32mgetting attribute values\e[0m\n");

        int snapId = ui.diffView->snapshotIds().at(m_nextSnapIndex);


        /* SCALAR NUMBERS */
        if(ui.diffView->sc1ValAttributes().size() > 0)
        {
            qDebug() << "chiedo scNum1Val per id " << snapId;
            m_snapDb->getAttValueList(snapId, AttInfo::scNum1);
        }
        if(ui.diffView->sc2ValAttributes().size() > 0)
        {
            qDebug() << "chiedo scNum2Val per id " << snapId;
            m_snapDb->getAttValueList(snapId, AttInfo::scNum2);
        }

        /* SCALAR STRINGS */
        if(ui.diffView->sc1ValStringAttributes().size() > 0)
            m_snapDb->getAttValueList(snapId, AttInfo::scStr1);

        if(ui.diffView->sc2ValStringAttributes().size() > 0)
            m_snapDb->getAttValueList(snapId, AttInfo::scStr2);

        /* SPECTRUM 1 and 2 VAL */
        if(ui.diffView->sp1ValAttributes().size() > 0)
            m_snapDb->getAttValueList(snapId, AttInfo::sp1);

        if(ui.diffView->sp2ValAttributes().size() > 0)
            m_snapDb->getAttValueList(snapId, AttInfo::sp2);

        /* IMAGE 1 and 2 VAL */
        if(ui.diffView->im1ValAttributes().size() > 0)
            m_snapDb->getAttValueList(snapId, AttInfo::Im1);

        if(ui.diffView->im2ValAttributes().size() > 0)
            m_snapDb->getAttValueList(snapId, AttInfo::Im2);


        m_nextSnapIndex++;
    }
}

void DiffDialog::selectAndClose()
{
    int snapId = ui.diffView->selectedSnapshotId();
    printf("selected snap %d\n", snapId);
    if(snapId > -1)
        emit selectSnapshot(snapId);
    close();
}

void DiffDialog::liveViewEnabled(bool enable)
{
    QList<DiffItemReader *> readerList = findChildren<DiffItemReader *>();
    if(!enable && !readerList.isEmpty())
    {
        foreach(DiffItemReader *reader, readerList)
            delete reader;
    }
    else if(readerList.isEmpty() && enable)
    {
        foreach(DiffAtt da, ui.diffView->attributeList())
        {
            DiffItemReader *reader = new DiffItemReader(this, da.id);
            connect(reader, SIGNAL(newData(int,QString,int,QString)), ui.diffView,
                    SLOT(liveDataAvailable(int,QString,int,QString)));
            reader->setSource(da.source);
        }
    }
}

void DiffDialog::filterTextChanged(const QString &text)
{
    TreeViewFilter *f = ui.diffView->findChild<TreeViewFilter *>();
    f->filter(text, 0);
}

void DiffDialog::filterOptionsChanged(int mode)
{
    ui.diffView->findChild<TreeViewFilter *>()->setFilterMode(static_cast<TreeViewFilter::FilterMode>(mode));
}

