#include "difftreeview.h"
#include "tlabelitemdelegate.h"
#include "modelview/treeitem.h"
#include "modelview/itemmodel.h"
#include "database/attdata.h"
#include "database/attinfo.h"
#include "modelview/treeviewfilter.h"
#include <QHeaderView>
#include <configuration.h>
#include <elettracolors.h>

#include "database/snapdb.h"
#include <macros.h>
#include <QTimer>
#include <math.h> /* fabs */

DiffTreeView::~DiffTreeView()
{
    pdelete("deleting DiffTreeView");
}

DiffTreeView::DiffTreeView(QWidget *parent) :
    QTreeView(parent)
{
    setRootIsDecorated(false);

#if QT_VERSION >= 0x050000
    header()->setSectionResizeMode(QHeaderView::Interactive);
#else
    header()->setResizeMode(QHeaderView::Interactive);
#endif

    QTimer *liveTimer = new QTimer(this);
    liveTimer->setObjectName("liveTimer");
    liveTimer->setInterval(1000);
    liveTimer->setSingleShot(false);
    connect(liveTimer, SIGNAL(timeout()), this, SLOT(updateLiveColumn()));

    setSelectionBehavior(QAbstractItemView::SelectColumns);

    connect(header(), SIGNAL(sectionClicked(int)), this, SLOT(selectColumn(int)));

#if QT_VERSION >= 0x050000
    header()->setSectionsClickable(true);
#else
    header()->setClickable(true);
#endif

    mPercent = -1.0;
    mHighlightDiffEnabled = false;
    mNoHighlightIfBelow = true;
    mNoHighlightThresh = 1e-9;
    mFilterOutNotHighlighted = false;
}

void DiffTreeView::setSnapshotIds(const QList<QVariant>& snaplist)
{
    QList<QVariant> headerItems = snaplist;
    for(int i = 0; i < headerItems.size(); i++)
    {
        /* i+1 because the first column will store the attribute name */
        m_snapIdColumnHash.insert(headerItems.at(i).toInt(), i + 1);
    }

    /* add column for attribute name */
    headerItems.push_front("Attribute name");
    /* add column for live mode */
    headerItems << "Live!";
    ItemModel *itemModel = new ItemModel(this, headerItems);
    setModel(itemModel);

    /* manage context selection */
    connect(selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
                this, SLOT(currentSelectionChanged(QModelIndex,QModelIndex)));
    //    setItemDelegateForColumn(headerItems.size() - 1, new TLabelItemDelegate(this));
}

/* this method only puts the attribute name as display role and the
 * attribute id in the UserRole.
 */
void DiffTreeView::update(const QList<AttInfo>& attInfoList)
{
    printf("updating attribute info list (%d els)\n", attInfoList.size());

    ItemModel *itemModel = static_cast<ItemModel *>(model());
    foreach(AttInfo ai, attInfoList)
    {
        int row = -1;
        for(int i = 0; i < itemModel->rowCount(); i++)
        {
            /* let's compare UserRole which contains an integer */
            if(itemModel->data(itemModel->index(i, 0), Qt::UserRole).toInt() == ai.id)
                return;
        }
        /* final false parameter not to emit dataChanged at every setData */
        if(row == -1) /* must create a new row at the bottom of the tree */
        {
            itemModel->insertRows(model()->rowCount(), 1);
            row = model()->rowCount() - 1;
        }

        itemModel->setData(model()->index(row, 0), ai.name, Qt::DisplayRole, false);
        itemModel->setData(model()->index(row, 0), ai.id, Qt::UserRole, false);

        /* attribute id is unique */
        m_attTypeInfoHash.insert(AttInfo::getType(ai.dataType, ai.format, ai.writable), ai.id);
        m_idRowHash.insert(ai.id, row);
    }
    /* trigger an update from the top left to the bottom right index */
    itemModel->update(itemModel->index(0, 0), itemModel->index(itemModel->rowCount() - 1, 1));
    header()->resizeSection(0, QHeaderView::ResizeToContents);

    emit attInfoReady();
}

void DiffTreeView::update(const QList<AttData>& values, AttInfo::Type)
{
    printf("update value list\n");
    ItemModel *itemModel = static_cast<ItemModel *>(model());
    foreach(AttData av, values)
    {
        int row = m_idRowHash.value(av.id);
        int column = m_snapIdColumnHash.value(av.idSnap);
        itemModel->setData(itemModel->index(row, column), av.setpoint, Qt::UserRole);
        itemModel->setData(itemModel->index(row, column), shortenedValue(av.setpoint), Qt::DisplayRole);
        itemModel->setData(itemModel->index(row, column), "<p>" + av.setpoint +
                           "</p>", Qt::ToolTipRole);

    }
    /* trigger an update from the top left to the bottom right index */
    itemModel->update(itemModel->index(0, 1), itemModel->
                      index(itemModel->rowCount() - 1, itemModel->columnCount() - 1));
    header()->resizeSections(QHeaderView::ResizeToContents);
}

QList<int> DiffTreeView::im1ValAttributes() const
{
    return m_attTypeInfoHash.values(AttInfo::Im1);
}

QList<int> DiffTreeView::im2ValAttributes() const
{
    return m_attTypeInfoHash.values(AttInfo::Im2);
}

QList<int> DiffTreeView::sc1ValAttributes() const
{
    return m_attTypeInfoHash.values(AttInfo::scNum1);
}

QList<int> DiffTreeView::sc2ValAttributes() const
{
    return m_attTypeInfoHash.values(AttInfo::scNum2);
}

QList<int> DiffTreeView::sp1ValAttributes() const
{
    return m_attTypeInfoHash.values(AttInfo::sp1);
}

QList<int> DiffTreeView::sp2ValAttributes() const
{
    return m_attTypeInfoHash.values(AttInfo::sp2);
}

QList<int> DiffTreeView::sc1ValStringAttributes() const
{
    return m_attTypeInfoHash.values(AttInfo::scStr1);
}

QList<int> DiffTreeView::sc2ValStringAttributes() const
{
    return m_attTypeInfoHash.values(AttInfo::scStr2);
}

QString DiffTreeView::shortenedValue(const QString& val) const
{
    QString ret = val;
    if(val.size() > 15)
    {
        ret.remove(15, 1000);
        ret += "...";
    }
    return ret;
}

void DiffTreeView::currentSelectionChanged(const QModelIndex& current, const QModelIndex& previous)
{
    Q_UNUSED(previous);
    printf("\e[1;33mcurrentSelectionChanged col %d row %d\e[0m\n\n", current.column(), current.row());
    if(current.column() == 0 || current.column() == model()->columnCount() - 1
            || current.column() < 0)
    {
        emit validSnapSelection(false);
    }
    else
        emit validSnapSelection(true);
}

QList<DiffAtt> DiffTreeView::attributeList() const
{
    QList<DiffAtt> ret;
    for(int i = 0; i < model()->rowCount(); i++)
    {
        DiffAtt da;
        da.source = model()->data(model()->index(i, 0), Qt::DisplayRole).toString();
        da.id = model()->data(model()->index(i, 0), Qt::UserRole).toInt();
        ret << da;
    }
    return ret;
}

void DiffTreeView::liveDataAvailable(int attId, const QString& val, int quality, const QString& message)
{
    bool ok1, ok2, hidden;
    QString value = val;
    ItemModel *m = static_cast<ItemModel *>(model());
    int valueColumn = m->columnCount() - 1;
    for(int i = 0; i < m->rowCount(); i++)
    {
        if(m->data(m->index(i, 0), Qt::UserRole).toInt() == attId)
        {
            m->setData(m->index(i, valueColumn), value, Qt::DisplayRole);
            m->setData(m->index(i, valueColumn), message, Qt::ToolTipRole);
            QColor c;
            switch(quality)
            {
            case Tango::ATTR_ALARM:
                c = Config::instance()->stateColor(Tango::FAULT);
                break;
            case Tango::ATTR_INVALID:
                c = KGRAY;
                break;
            case Tango::ATTR_VALID:
                c = Config::instance()->stateColor(Tango::RUNNING);
                break;
            case Tango::ATTR_WARNING:
                c = Config::instance()->stateColor(Tango::ALARM);
                break;
            case Tango::ATTR_CHANGING:
                c = Config::instance()->stateColor(Tango::MOVING);
                break;
            default:
                c = Config::instance()->stateColor(Tango::OFF);
                break;
            }
            c = c.darker(200);
            m->setData(m->index(i, valueColumn), c, Qt::ForegroundRole);
//            m->setData(m->index(i, valueColumn), Qt::black, Qt::BackgroundRole);
            QFont t = m->data(m->index(i, valueColumn), Qt::FontRole).value<QFont>();
            t.setBold(true);
            m->setData(m->index(i, valueColumn), t, Qt::FontRole);

            value = value.remove(QRegExp("\\s*\\[.*\\]\\s*"));
            double curVal = value.toDouble(&ok2);
           // printf("value %s --> %f\n", qstoc(value), curVal);

            for(int c = 1; c < valueColumn; c++)
            {
                double snapVal = m->data(m->index(i, c), Qt::UserRole).toString().toDouble(&ok1);
                double maxdelta = snapVal * mPercent / 100.0;
                bool noHighlight = mNoHighlightIfBelow && (fabs(snapVal) < mNoHighlightThresh && fabs(curVal) < mNoHighlightThresh);
                if(( mHighlightDiffEnabled && ok1 && ok2 && fabs(snapVal - curVal) > fabs(maxdelta))
                        && !noHighlight)
                {
                    m->setData(m->index(i, c), KRED, Qt::BackgroundRole);
                }
                else
                    m->setData(m->index(i, c), QColor(Qt::white), Qt::BackgroundRole);
            }
        }



    }
    TreeViewFilter *filter = findChild<TreeViewFilter *>();
    if(filter)
        filter->filter(filter->filterText(), 0);
    for(int i = 0; i < m->rowCount() && !isRowHidden(i, QModelIndex()); i++)
    {
        hidden = (mFilterOutNotHighlighted && m->data(m->index(i, 1), Qt::BackgroundRole).value<QColor>() == QColor(Qt::white));
        setRowHidden(i, QModelIndex(), hidden);
    }
}

void DiffTreeView::updateLiveColumn()
{
    int valueColumn = model()->columnCount() - 1;
    static_cast<ItemModel *>(model())->update(model()->index(0, 0),
          model()->index(model()->rowCount() - 1, valueColumn));
    header()->resizeSection(valueColumn, QHeaderView::ResizeToContents);
}

int DiffTreeView::selectedSnapshotId() const
{
    QItemSelection itemSelection = selectionModel()->selection();
    ItemModel *itemModel = static_cast<ItemModel *>(model());
    QList <QModelIndex> sel = itemSelection.indexes();
    if(sel.size() && sel.first().column() != 0 && sel.first().column() != model()->columnCount() - 1)
    {
        return itemModel->headerData(sel.first().column(), Qt::Horizontal, Qt::DisplayRole).toInt();
    }
    return -1;
}

void DiffTreeView::selectColumn(int c)
{
    for(int i = 0; i < model()->columnCount(); i++)
    {
        QItemSelection s(model()->index(0, i), model()->index(model()->rowCount() - 1, i));
        selectionModel()->select(s, QItemSelectionModel::Deselect);
    }
    if(c != model()->columnCount() - 1 && c != 0)
    {
        QItemSelection s(model()->index(0, c), model()->index(model()->rowCount() - 1, c));
        selectionModel()->select(s, QItemSelectionModel::Select);
        emit validSnapSelection(true);
    }
    else
        emit validSnapSelection(false);
}

void DiffTreeView::liveViewEnabled(bool en)
{
    QTimer *liveTimer = findChild<QTimer *>("liveTimer");
    en ? liveTimer->start() : liveTimer->stop();
    if(!en)
    {
        ItemModel *m = static_cast<ItemModel *>(model());
        int valueColumn  = m->columnCount() - 1;
        for(int i = 0; i < m->rowCount(); i++)
        {
            m->setData(m->index(i, valueColumn), "-", Qt::DisplayRole);
            m->setData(m->index(i, valueColumn), "-", Qt::ToolTipRole);
            m->setData(m->index(i, valueColumn), QColor(Qt::black), Qt::ForegroundRole);
            m->setData(m->index(i, valueColumn), QColor(Qt::white), Qt::BackgroundRole);
//            m->setData(m->index(i, valueColumn), Qt::white, Qt::BackgroundRole);
            QFont t = m->data(m->index(i, valueColumn), Qt::FontRole).value<QFont>();
            t.setBold(false);
            m->setData(m->index(i, valueColumn), t, Qt::FontRole);
        }
    }
}

void DiffTreeView::setNoHightlightThresh(const QString &threshAsStr)
{
    bool ok;
    double th = threshAsStr.toDouble(&ok);
    if(ok)
        this->mNoHighlightThresh = th;
}


