#include "itemmodel.h"
#include "treeitem.h"
#include <macros.h>

ItemModel::ItemModel(QObject *parent, const QList<QVariant>& rootData) :
    QAbstractItemModel(parent)
{
    m_rootItem  = new TreeItem(rootData);
    m_supportedRoles << Qt::DisplayRole << Qt::ToolTipRole << Qt::UserRole
                     << Qt::DecorationRole << Qt::ForegroundRole
                     << Qt::BackgroundRole << Qt::FontRole;
}

ItemModel::~ItemModel()
{
    delete m_rootItem;
}

QVariant ItemModel::data(const QModelIndex &index, int role) const
{
    if (!index.isValid())
        return QVariant();

    if (!m_supportedRoles.contains((Qt::ItemDataRole)role))
        return QVariant();

    TreeItem *item = static_cast<TreeItem*>(index.internalPointer());

    return item->data(index.column(), (Qt::ItemDataRole) role);
}

void ItemModel::update(const QModelIndex& from, const QModelIndex& to)
{
    emit dataChanged(from, to);
}

bool ItemModel::setData(const QModelIndex &idx, const QVariant &value, Qt::ItemDataRole role, bool update)
{
    if(!idx.isValid() || !m_supportedRoles.contains(role))
    {
        perr("ItemModel: setData() fails: idx valid %d role %d", idx.isValid(), role);
        return false;
    }
    TreeItem *item = static_cast<TreeItem*>(idx.internalPointer());
    bool ret = item->setData(value, idx.column(), role);
    if(ret)
    {
        if(update)
        {
           // printf("\e[1;32m emetto data changed: %d %d\e[0m\n", idx.row(), idx.column());
            //printf("parent in data changed idx %s / data changed in \"%s\"\n", qstoc(data(idx.parent(), role).toString()),
            //      qstoc(data(idx, role).toString()));
            emit dataChanged (idx , idx);
        }
    }
    else
        perr("failed to setData %s, index.column %d", qstoc(value.toString()), idx.column());
    return ret;
}

Qt::ItemFlags ItemModel::flags(const QModelIndex &index) const
{
    if (!index.isValid())
        return 0;

    return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant ItemModel::headerData(int section, Qt::Orientation orientation, int role) const
{
    if (orientation == Qt::Horizontal && role == Qt::DisplayRole && columnCount() > section)
        return m_rootItem->data(section, (Qt::ItemDataRole)role);
    return QVariant();
}

QModelIndex ItemModel::index(int row, int column, const QModelIndex &parent) const
{
    if (!hasIndex(row, column, parent))
        return QModelIndex();

    TreeItem *parentItem;

    if (!parent.isValid())
        parentItem = m_rootItem;
    else
        parentItem = static_cast<TreeItem*>(parent.internalPointer());

    TreeItem *childItem = parentItem->child(row);
    if (childItem)
        return createIndex(row, column, childItem);
    else
        return QModelIndex();
}

QModelIndex ItemModel::parent(const QModelIndex &index) const
{
    if (!index.isValid())
        return QModelIndex();

    TreeItem *childItem = static_cast<TreeItem*>(index.internalPointer());
    TreeItem *parentItem = childItem->parent();

    if (parentItem == m_rootItem)
        return QModelIndex();

    return createIndex(parentItem->row(), 0, parentItem);

}

int ItemModel::rowCount(const QModelIndex &parent) const
{
    TreeItem *parentItem;

    if (!parent.isValid())
        parentItem = m_rootItem;
    else
        parentItem = static_cast<TreeItem*>(parent.internalPointer());

    return parentItem->childCount();
}

int ItemModel::columnCount(const QModelIndex &parent) const
{
    if (parent.isValid())
        return static_cast<TreeItem*>(parent.internalPointer())->columnCount();
    else
        return m_rootItem->columnCount();

}


void ItemModel::setupModelData(const QStringList &, TreeItem *)
{

}

bool ItemModel::insertRows(int row, int count, const QModelIndex &parent)
{
    beginInsertRows(parent, row, row + count);

    TreeItem *parentItem;
    if(!parent.isValid())
        parentItem = m_rootItem;
    else
    {
        parentItem = static_cast<TreeItem*>(parent.internalPointer());
//        printf("\e[1;33mparent is valid index (%d,%d) and is %s\e[0m\n",
//               parent.row(), parent.column(), qstoc(parentItem->data(0, Qt::DisplayRole).toString()));
    }

    for(int i = 0; i < count; i++)
    {
        QList<QVariant > data;
        for(int c = 0;  c < columnCount(parent); c++)
            data << "-";
        TreeItem* mti = new TreeItem(data, parentItem);
        parentItem->appendChild(mti);
    }
    if(count)
    {
        endInsertRows();
        return true;
    }
    else
        return false;
}

bool ItemModel::removeRows(int row, int count, const QModelIndex &parent)
{
   beginRemoveRows(parent, row, row + count);

   TreeItem *item;
   if(parent == QModelIndex())
   {
       item = m_rootItem;
   }
   else
   {
       item = static_cast<TreeItem *>(parent.internalPointer());
   }
   item->removeChildren(row, count);

   endRemoveRows();
   return true;
}



