#include "restore.h"
#include "variantfromstring.h"
#include "writer.h"
#include "../attributedatatreeview.h"
#include "../log/logger.h"
#include <QAbstractItemModel>
#include <QAbstractItemView>
#include <QVariant>
#include <macros.h>

Restore::Restore(QObject *parent, Logger *logger) :
    QObject(parent)
{
    mLogger = logger;
    init();
}

void Restore::init()
{
    m_step = 0;
    m_tangoErrors.clear();
    deleteWriters();
}

void Restore::restore(AttributeDataTreeView *view)
{
    emit started();
    QAbstractItemModel *model = view->model();
    /* initialize data structures before new restore */
    init();

    emit status("restoring tango");

    int restoreColumn = view->getRestoreColumnIndex();
    int attributeNameColumn = view->getColumnIndex(AttributeDataTreeView::ColumnAttr);
    int columnFormat = view->getColumnIndex(AttributeDataTreeView::ColumnFormat);
    int columnType = view->getColumnIndex(AttributeDataTreeView::ColumnType);
    int columnWritable = view->getColumnIndex(AttributeDataTreeView::ColumnWritable);

    pinfo("restore: %d attributes", model->rowCount());
    for(int i = 0; i < model->rowCount(); i++)
    {
        /* check if data stored in model is appropriate for restore.
         * Are excluded:
         * 1. read only attributes;
         * 2. attributes with NULL set point.
         * 3. Tango::Image attributes.
         */
        bool itemIsRestorable, itemSelected;
        QString reason;
        QString target = model->data(model->index(i, attributeNameColumn),
                        Qt::DisplayRole).toString();

        QModelIndex setPointIndex = model->index(i, restoreColumn);

        Tango::AttrDataFormat fmt =
                static_cast<AttrDataFormat>(model->data(model->index(i, columnFormat),
                Qt::UserRole).toInt());

        /* NOTE: set point values are taken from the UserRole data, not from the DisplayRole, which
         * may be truncated.
         */
        itemIsRestorable = itemRestorable(
            model->data(setPointIndex, Qt::UserRole).toString(),
            model->data(model->index(i, columnType), Qt::UserRole).toInt(),
            model->data(model->index(i, columnFormat), Qt::UserRole).toInt(),
            model->data(model->index(i, columnWritable), Qt::UserRole).toInt(),
            reason);

        QItemSelection itemSelection = view->selectionModel()->selection();
        itemSelected = itemSelection.contains(setPointIndex);
        m_total = itemSelection.size();
        if(itemIsRestorable && itemSelected)
        {
            QString valueToRestore = model->data(model->index(i, restoreColumn),
                            Qt::UserRole).toString();

            printf("\e[1;32m******** RESTORING \"%s\" on \"%s\"\e[0m\n", qstoc(target), qstoc(valueToRestore));


            Writer *writer = new Writer(this, target, qobject_cast<AttributeDataTreeView *>(view)->currentSnapId());
            connect(writer, SIGNAL(targetExecuted()), this, SLOT(writerTargetExecuted()));
            connect(writer, SIGNAL(connectionFailed(QString,QString)), this,
                    SLOT(error(QString,QString)));
            connect(writer, SIGNAL(executionFailed(QString,QString)), this,
                    SLOT(error(QString,QString)));
            writer->setTargets(target);
            printf("calling \e[1;32mexecute\e[0m with makeVariant\e[0m\n");
            // variant from string convert string to QVariant for execute
            writer->execute(VariantFromString(valueToRestore, fmt).value());
        }
        else if(!itemIsRestorable && itemSelected) /* selected but impossible to restore: error */
            error(target, reason);
        /* if not restorable, but neither selected, no problem */
    }
}

void Restore::clearErrors()
{
    m_tangoErrors.clear();
}

void Restore::error(const QString& target, const QString& msg)
{
    Writer * writer = qobject_cast<Writer *>(sender());
    if(mLogger != NULL && writer != NULL)
        mLogger->addTangoWriteError(target, msg, writer->snapId());

    m_tangoErrors.insert(target, msg);
    m_step++;
    printf("\e[1;33merror() emitting progress %d/%d\e[0m\n", m_step, m_total);
    emit progress(m_step, m_total);
    checkForFinished();
}

void Restore::writerTargetExecuted()
{
    m_step++;
    emit progress(m_step, m_total);
    checkForFinished();
}

void Restore::checkForFinished()
{
    if(m_step == m_total)
    {
        printf("\e[1;34memittied finished tot %d err %d\e[0m\n", m_total, m_tangoErrors.size());
        emit finished();
        emit status("restore finished");
    }
}

void Restore::deleteWriters()
{
    foreach(Writer *w, findChildren<Writer *>())
        delete w;
}

bool Restore::itemRestorable(const QString& setPoint,
                             int type,
                             int format,
                             int writable,
                             QString& reason) const
{

    Tango::AttrDataFormat fmt = static_cast<Tango::AttrDataFormat>(format);
    if(fmt == Tango::IMAGE)
    {
        reason = "Image format unsupported";
        return false;
    }
    Tango::AttrWriteType wri = static_cast<Tango::AttrWriteType>(writable);
    if(wri == Tango::READ)
    {
        reason = "attribute is READ ONLY";
        return false;
    }
    if(setPoint == "NULL" || setPoint.isEmpty() || setPoint == "-")
    {
        reason = "set point is NULL";
        return false;
    }
    Tango::CmdArgType adt = static_cast<CmdArgType>(type);
    if(adt >= DEV_ENCODED)
    {
        reason = "attribute data type unsupported";
        return false;
    }
    return true;
}
