#include "snapselectionmanager.h"
#include "attributedatatreeview.h"
#include <QAbstractItemView>
#include <QItemSelectionModel>
#include <QFile>
#include <QTextStream>
#include <QDir>
#include <QSettings>
#include <QItemSelection>
#include <QtDebug>
#include <macros.h>

SnapSelectionManager::SnapSelectionManager(AttributeDataTreeView *parent) :
    QObject(parent)
{
    m_view = parent;
    m_currentId = -1;
    QString path = filepath();
    QDir etcDir = QDir(path);
    if(!etcDir.exists() && !etcDir.mkpath(path))
            m_errorMessage = "Failed to create directory " + path;
    if(!m_errorMessage.isEmpty())
        emit error("Snapshot selection manager", m_errorMessage);
    qDebug() << __FUNCTION__ << "SnapSelectionManager created!";
}

SnapSelectionManager::~SnapSelectionManager()
{
    qDebug() << __FUNCTION__ << "SnapSelectionManager DESTROYED!";
}

void SnapSelectionManager::loadFromId(int id)
{
    if(id >= 0)
    {
        m_currentId = id;
        bool selectAll = false;
        QList<int> attIdList;
        QFile file;
        bool ok;
        file.setFileName(filename(id));
        /* if file is not found, then the snap might be new */
        if(!file.open(QIODevice::ReadOnly | QIODevice::Text))
        {
            selectAll = true;
            printf("\e[1;31mfile \"%s\" doesn't exist selecting all\e[0m\n", qstoc(file.fileName()));
        }
        else
        {
            m_errorMessage.clear();
            printf("\e[1;32mfile \"%s\" reading from it\e[0m\n", qstoc(file.fileName()));
            while(!file.atEnd())
            {
                QByteArray line = file.readLine();
                line.remove(line.length() - 1, 1); /* remove '\n' */
                int attID = line.toInt(&ok);
                if(!ok)
                {
                    m_errorMessage = QString("Malformed line %1 in file \"%2\"").arg(QString(line)).
                            arg(file.fileName());
                    emit error("Snapshot selection manager", m_errorMessage);
                }
                else
                    attIdList << attID;
            }
            /* close file */
            file.close();
        }
        QItemSelectionModel *selectionModel = m_view->selectionModel();
        QAbstractItemModel *itemModel = m_view->model();
        for(int i = 0; i < itemModel->rowCount(); i++)
        {
            int attId = itemModel->data(itemModel->index(i, 0), Qt::UserRole).toInt();
            QItemSelectionModel::SelectionFlag sf;
            if(attIdList.contains(attId) || selectAll)
                sf = QItemSelectionModel::Select;
            else
                sf = QItemSelectionModel::Deselect;

            selectionModel->select(QItemSelection(itemModel->index(i, 0),
                       itemModel->index(i, itemModel->columnCount() - 1)), sf);
        }
    }
}

QString SnapSelectionManager::filename(int id) const
{
    QString filena = filepath() + QString::number(id);
    if(m_view->getMode() == AttributeDataTreeView::Hdb)
        filena += "_ctx";
    /* else: as before to avoid breaking compatibility */
    return filena;
}

QString SnapSelectionManager::filepath() const
{
    QSettings s;
    QString path = s.value("SNAP_SELECTION_FILENAME",
        QDir::homePath() + "/etc/saverestore/restore/snapselections/").toString();
    if(!path.endsWith('/'))
        path += '/';
    return path;
}


void SnapSelectionManager::saveSelection()
{
    QFile file(filename(m_currentId));
    qDebug() << __FUNCTION__ << file.fileName() << m_view->selectionModel()->selectedIndexes();
    if(!file.open(QIODevice::WriteOnly | QIODevice::Text))
    {
        m_errorMessage = QString("Error opening \"%1\" (Write): %2")
                .arg(file.fileName()).arg(file.errorString());
        emit error("Snapshot selection manager", m_errorMessage);
    }
    else
    {
        QTextStream out(&file);
        QModelIndexList selectedIdxs = m_view->selectionModel()->selectedIndexes();
        QAbstractItemModel *model = m_view->model();
        for(int i = 0; i < selectedIdxs.size(); i++)
        {
            if(selectedIdxs.at(i).column() == 0)
            {
                int row = selectedIdxs.at(i).row();
                int attId = model->data(model->index(row, 0), Qt::UserRole).toInt();
                qDebug() << __FUNCTION__ << "saving on " << file.fileName() << row << "Att id " << attId;
                out << QString("%1\n").arg(attId);
            }
        }
        /* close file */
        file.close();
    }
}

void SnapSelectionManager::itemClicked(const QModelIndex&)
{
    saveSelection();
}

void SnapSelectionManager::deselectReadOnlyAndNullItems()
{
    QAbstractItemModel *model = m_view->model();
    int columnWritable = m_view->getColumnIndex(AttributeDataTreeView::ColumnWritable);
    int columnSetPoint = m_view->getColumnIndex(AttributeDataTreeView::ColumnSetPoint);
    for(int i = 0; i < model->rowCount(); i++)
    {
        int wri = model->data(model->index(i, columnWritable), Qt::UserRole).toInt();
        Tango::AttrWriteType writable = static_cast<Tango::AttrWriteType>(wri);
        QString setPt = model->data(model->index(i, columnSetPoint)).toString();
        if(writable == Tango::READ || setPt == "NULL")
        {
            QItemSelection selection(model->index(i, 0), model->index(i, model->columnCount() - 1));
            m_view->selectionModel()->select(selection, QItemSelectionModel::Deselect);
        }
    }
}
