#include "collector.h"
#include "reader.h"

/* to get AttInfo::Type from writable data format and data type */
#include "../database/attinfo.h"
#include "../log/logger.h"

#include <QTWatcher>
#include <QtDebug>

Collector::Collector(QObject *parent, Logger *logger) :
    QObject(parent)
{
    mLogger = logger;
}

void Collector::deleteReaders()
{
    foreach(Reader *r, findChildren<Reader *>())
        delete r;
}

void Collector::init()
{
    m_totSources = 0;
    m_remainingSources.clear();
    m_attErrors.clear();
    m_attMessages.clear();
    m_attValues.clear();
}

void Collector::read(const QAbstractItemModel *model, const int snapId)
{
    init();
    for(int i = 0; i < model->rowCount(); i++)
    {
        Tango::CmdArgType dt = (Tango::CmdArgType )
                model->data(model->index(i, 3), Qt::UserRole).toInt();
        Tango::AttrWriteType wt = (Tango::AttrWriteType)
                model->data(model->index(i, 5), Qt::UserRole).toInt();
        Tango::AttrDataFormat fmt = (Tango::AttrDataFormat)
                model->data(model->index(i, 4), Qt::UserRole).toInt();

        AttInfo::Type itype = AttInfo::getType(dt, fmt, wt);

        int attId = model->data(model->index(i, 0), Qt::UserRole).toInt();

        QString source = model->data(model->index(i, 0), Qt::DisplayRole).toString();
        /* duplicated sources? No thanks... it shouldn't happen, but in a test context
         * there were duplicates. m_remainingSources is a QSet.
         */
        m_remainingSources.insert(source); /* first, populate set */
        Reader *reader = new Reader(this, itype, wt, attId, source, snapId);
        connect(reader, SIGNAL(connectionOk(bool)), this, SLOT(readerConnectionOk(bool)));
        connect(reader, SIGNAL(dataAvailable(bool)), this, SLOT(readerDataAvailable(bool)));
        connect(reader, SIGNAL(connectionErrorMessage(QString)), this, SLOT(slotConnectionErrorMessage(QString)));
    }
    m_totSources = m_remainingSources.size();

    /* after populating set, setSource on readers.
     * readerConnectionOk() might be immediately invoked in some cases,
     * so the set must be filled first.
     */
    foreach(Reader *reader, findChildren<Reader *>())
        reader->setSource(reader->objectName());
}

QList<AttInfo::Type> Collector::types() const
{
    return m_attValues.keys();
}

void Collector::readerConnectionOk(bool ok)
{
    Reader *reader = static_cast<Reader *>(sender());
    if(!ok)
    {


        reader->disconnect(this);

        m_attErrors.insert(reader->objectName(), reader->errorMessage());
        /* use object name instead of source */
        m_remainingSources.remove(reader->objectName());
        /* invalid attValue */
        m_attValues.insert(reader->attValue().type, reader->attValue());
        emit progress(m_totSources - m_remainingSources.size(), m_totSources);

        if(m_remainingSources.isEmpty()) /* all sources read, or received an error */
            emit finished();
    }
}

void Collector::readerDataAvailable(bool ok)
{
    Reader *reader = static_cast<Reader *>(sender());
    reader->disconnect(this);
    /* immediately stop reading */
    reader->unsetSource();
    /* insert the attribute value into the map */
    m_attValues.insert(reader->attValue().type, reader->attValue());

    if(!ok)
    {
        if(mLogger != NULL)
            mLogger->addTangoReadError(reader->objectName(), reader->errorMessage(), reader->snapId());
        m_attErrors.insert(reader->objectName(), reader->errorMessage());
    }
    else
        m_attMessages.insert(reader->objectName(), reader->message());

    m_remainingSources.remove(reader->objectName());
    emit progress(m_totSources - m_remainingSources.size(), m_totSources);

    if(m_remainingSources.isEmpty()) /* all sources read, or received an error */
        emit finished();
}

void Collector::slotConnectionErrorMessage(const QString& msg)
{
    printf("Collector::slotConnectionErrorMessage: %s\n", qstoc(msg));
    Reader *reader = static_cast<Reader *>(sender());
    if(mLogger != NULL)
        mLogger->addTangoReadError(reader->objectName(), reader->errorMessage(), reader->snapId());

}
