#include "reader.h"
#include <macros.h>

Reader::~Reader()
{
    pdelete("deleting reader for att id %d", m_attValue.id);
    setRefreshMode(POLLED_REFRESH);
    setPeriod(3600 * 24 * 30);
}

Reader::Reader(QObject *parent, AttInfo::Type t,
               Tango::AttrWriteType writable,
               int attId, const QString& objName,
               const int snapId) :
    QObject(parent), QTangoComProxyReader(this)
{
    m_attValue.idSnap = -1;
    m_attValue.id= attId;
    m_attValue.dimX = 1;
    m_attValue.type = t;
    m_attValue.writable = writable;
    m_attValue.value = m_attValue.setpoint = "NULL";
    m_snapId = snapId;

    setObjectName(objName);
    connect(qtangoComHandle(), SIGNAL(newData(const TVariant&)), this,
            SLOT(refresh(const TVariant&)));
    connect(qtangoComHandle(), SIGNAL(connectionOk(bool)), this, SLOT(slotConnectionOk(bool)));
    connect(qtangoComHandle(), SIGNAL(connectionErrorMessage(QString)), this,
            SLOT(slotConnectionErrorMessage(QString)));
}

void Reader::refresh(const TVariant &v)
{
    bool dataConversionError = false;

    m_errorMessage.clear();
    m_message = v.message();

    if(v.quality() == Tango::ATTR_INVALID)
    {
        /* m_attValue will remain invalid.
         * Collector will take care of invalid AttValue objects.
         */
        m_attValue.value = "NULL";
        m_attValue.value = "NULL";
        m_errorMessage = "Read error: "  + m_message;
    }
    else
    {
        if(m_attValue.type == AttInfo::scNum1 || m_attValue.type == AttInfo::scStr1)
        {
            if(v.canConvertToString())
            {
                m_attValue.value = v.toString(m_attValue.writable == Tango::READ, true);
                /* if a localized conversion takes place, the string may contain commas
                 * to separate thousands. This is not compatible with database
                 * data storing.
                 */
                if(m_attValue.value.contains(","))
                    m_attValue.value.remove(",");
            }
            else
                dataConversionError = true;
        }
        else if(m_attValue.type == AttInfo::scNum2 || m_attValue.type == AttInfo::scStr2)
        {
            if(v.canConvertToString())
            {
                m_attValue.value = v.toString(true, true); /* read, forceNoUnit */
                if(m_attValue.value.contains(','))
                    m_attValue.value.remove(",");
                /* set point */
                m_attValue.setpoint = v.toString(false, true); /* set point, forceNoUnit */
                if(m_attValue.setpoint.contains(','))
                    m_attValue.setpoint.remove(",");
            }
            else
                dataConversionError = true;
            printf("Reader.refresh: got %s read val %s write val %s\n", qstoc(v.tangoPoint()), qstoc(m_attValue.value), qstoc(m_attValue.setpoint));
        }
        else if(m_attValue.type == AttInfo::sp1)
        {
            if(v.canConvertToStringVector())
            {
                QStringList values = v.toStringVector(m_attValue.writable == Tango::READ).toList();
                /* remove commas from strings, if present */
                for(int i = 0; i < values.size(); i++)
                {
                    if(values[i].contains(','))
                        values[i].remove(',');
                }
                m_attValue.dimX = values.size();
                m_attValue.value = values.join(",");
            }
            else
                dataConversionError = true;
        }
        else if(m_attValue.type == AttInfo::sp2)
        {
            if(v.canConvertToStringVector())
            {
                QStringList values = v.toStringVector().toList();
                /* remove commas from strings, if present */
                for(int i = 0; i < values.size(); i++)
                {
                    if(values[i].contains(','))
                        values[i].remove(',');
                }
                m_attValue.value = values.join(",");
                m_attValue.dimX = values.size();
                /* set point */
                values = v.toStringVector(false).toList();
                /* remove commas from strings, if present */
                for(int i = 0; i < values.size(); i++)
                {
                    if(values[i].contains(','))
                        values[i].remove(',');
                }
                m_attValue.setpoint = values.join(",");
            }
            else
                dataConversionError = true;
        }
    }

    if(dataConversionError)
        m_errorMessage = "Wrong data type";
    else
        emit dataAvailable(m_errorMessage.isEmpty());

}

void Reader::slotConnectionOk(bool ok)
{
    if(!ok)
    {
        emit connectionError();
        perr("disconnecting source \"%s\" after connection failure", qstoc(source()));
        unsetSource();
    }
    emit connectionOk(ok);
}

void Reader::slotConnectionErrorMessage(const QString& msg)
{
    this->m_errorMessage = msg;
    printf("Reader: slot connection error msg %s\n", qstoc(msg));
    emit connectionErrorMessage(msg);
}
