#include "save.h"
#include "../database/snapinsertquery.h"
#include "../database/addsnapshotquery.h"
#include <macros.h>

Save::Save(QObject *parent, SnapDb *snapDb, Logger *logger) :
    QObject(parent)
{
    m_snapDb = snapDb;
    m_contextId = -1;
    m_model = NULL;

    /* adds a snapshot id, comment, date time into the snapshot table */
    AddSnapshotQuery *addSnapQuery = new AddSnapshotQuery(this);
    snapDb->registerDbQuery("AddSnapshotQuery", addSnapQuery);
    connect(addSnapQuery, SIGNAL(error(QString,QString)), this, SIGNAL(dbWriteError(QString,QString)));
    connect(addSnapQuery, SIGNAL(snapAdded(bool, int)), this, SLOT(saveData(bool, int)));

    /* data query: inserts quantities read from tango in t_xx_xx_val tables */
    SnapInsertQuery *snapInsertQuery = new SnapInsertQuery(this);
    snapDb->registerDbQuery("SnapInsertQuery", snapInsertQuery);
    connect(snapInsertQuery, SIGNAL(queryExecuted(const QString&, bool, const QString&)),
            this, SLOT(dataQueryExecuted(const QString&, bool, const QString&)));

    /* collects data from the tango devices */
    Collector *collector = new Collector(this, logger);
    connect(collector, SIGNAL(finished()), this, SLOT(collectorFinished()));
    connect(collector, SIGNAL(progress(int,int)), this, SIGNAL(readProgress(int,int)));
}

void Save::save(const QString& comment)
{
    if(m_contextId > 0)
    {
        emit started();

        pinfo("Save::save() adding snapshot information to snapshot table for context %d",
              m_contextId);

        AddSnapshotQuery *addSnapQuery = static_cast<AddSnapshotQuery *>
                (m_snapDb->dbQueryInterface("AddSnapshotQuery"));

        /* reset() must be called to clear snap query fields such as comment, ctx id
         * and error buffer.
         */
        addSnapQuery->init();

        /* provide a default comment if the user didn't type one */
        if(comment.isEmpty())
            addSnapQuery->setComment(QString("Snap of context %1 ").arg(m_contextId) +
                    QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss"));
        else
            addSnapQuery->setComment(comment);
        addSnapQuery->setContextId(m_contextId);
        emit status("Adding snap info to snap table");
        addSnapQuery->execute(m_contextId);
    }
}

void Save::saveData(bool ok, int snapId)
{
    if(ok && m_model)
    {
        printf("\e[1;35m*** \e[0m saveData entered\n");
        m_queryCnt = 0;
        m_activeQueries.clear();
        m_dbErrorCnt = 0;

        pinfo("Save::saveData(): collecting data from tango devices");

        /* important: clear all data inside the current SnapInsertQuery */
        SnapInsertQuery * snapInsertQuery = static_cast<SnapInsertQuery *>
                (m_snapDb->dbQueryInterface("SnapInsertQuery"));
        snapInsertQuery->setSnapshotId(snapId);
        snapInsertQuery->reset();

        emit status("Reading tango devices");
        Collector *collector = findChild<Collector *>();
        collector->read(m_model, snapId); /* Collector's read reinitializes data structures */
    }
}

void Save::collectorFinished()
{
    Collector *collector = findChild<Collector *>();
    QMultiMap<AttInfo::Type, AttData> attValues = collector->attValues();
    m_readErrorCnt = collector->attErrors().size();

    SnapInsertQuery *snapInsertQuery =
            static_cast<SnapInsertQuery *>(m_snapDb->dbQueryInterface("SnapInsertQuery"));
    emit status("Saving data into the snap db");
    foreach(AttInfo::Type t, attValues.uniqueKeys())
    {
        snapInsertQuery->setData(t, attValues.values(t));
        snapInsertQuery->execute(t);
        m_activeQueries.insert(t, snapInsertQuery);
        m_queryCnt++;
    }
    if(!m_queryCnt) /* all failed! */
    {
        printf("\e[1;36m===============================\n\e[0memitting finished!!!\e[0m\n");
        emit finished(m_contextId, snapInsertQuery->snapshotId());
        emit finished(m_contextId);
    }
}

void Save::dataQueryExecuted(const QString& q, bool success, const QString& errorMsg)
{
    SnapInsertQuery *snapInsertQuery = static_cast<SnapInsertQuery *>(sender());
    m_activeQueries.remove(m_activeQueries.key(snapInsertQuery));

    if(!success)
    {
        emit dbWriteError(q, errorMsg);
        perr("Save::queryExecuted() error: \"%s\" \"%s\"", qstoc(q), qstoc(errorMsg));
        m_dbErrorCnt++;
    }

    emit dbWriteProgress(m_queryCnt - m_activeQueries.size(), m_queryCnt);

    if(m_activeQueries.isEmpty())
    {
        findChild<Collector *>()->deleteReaders();
        printf("\e[1;33m===============================\n\e[0memitting finished!!!\e[0m\n");
        emit finished(m_contextId, snapInsertQuery->snapshotId());
        emit finished(m_contextId);
    }
}

unsigned Save::readErrorCount() const
{
    return findChild<Collector *>()->attErrors().size();
}

QHash<QString, QString> Save::attErrors() const
{
    return findChild<Collector *>()->attErrors();
}

