#include "saverestore.h"
#include "snapshotlistcontainer.h"
#include "snapinfoviewdbhelper.h"
#include "groupboxbackintime.h"
#include "snapshotview.h"
#include "attributedatatreeview.h"
#include "timemachineattributedatatreeview.h"
#include "ui_saverestore.h"
#include "database/snapdb.h"
#include "database/attdata.h"
#include "database/attinfoquery.h"
#include "save/save.h"
#include "restore/snapselectionmanager.h"
#include "led.h"
#include "about.h"
#include "progressbar.h"
#include "uihelper.h"
#include "restore/restore.h"
#include "diff/diffdialog.h"
#include "selection/selectiondialog.h"
#include "selection/selection.h"
#include "settings/settingsdialog.h"
#include "settings/settings.h"
#include "extraction/extractor.h"
#include "selectfromdiarydialog.h"
#include "log/logger.h"
#include <QMetaType>
#include <QTimer>
#include <action_factory.h>
#include <QtDebug>
#include <QSplitter>
#include <QDateTimeEdit>

SaveRestore::SaveRestore(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::SaveRestore)
{
    QStringList args = qApp->arguments();
    int argc = args.size();
    bool timeMachine = false;
    mSelectedContextId = -1;
    /* disable TLog/TUtil::addLog for all sources targets that
     * will be created by SaveRestore.
     */
    ActionFactory::actionFactory()->setLoggingDisabled(true);

    ActionFactory::actionFactory()->setDefaultRefreshMode(POLLED_REFRESH);

    ui->setupUi(this);

    /* log directory from command line arguments */
    QString logDir = "";
    if(args.contains("--log-dir") && argc > args.indexOf("--log-dir") + 1)
        logDir = args.at(qApp->arguments().indexOf("--log-dir") + 1);
    if(args.contains("--time-machine"))
        timeMachine = true;
    mLogger = new Logger(logDir);

    mSetupCommon();

    if(timeMachine)
        mSetupTimeMachine();
    else
        mSetupSaveRestore();

    mSetupCommon2();
    /* expand context view at startup */
    ui->cbExpandContextView->setChecked(true);
}

void SaveRestore::mSetupCommon()
{
    ui->gbErrors->setHidden(true);

    SnapDb *snapDb = new SnapDb(this);
    /* UI HELPER: manages interface interaction with user, such as
     * shows/hides enables/disables part of the user interface
     * according to determined user actions. Create UiHelper here, it
     * will be mainly setup in mSetupCommon2
     */
    UiHelper *uiHelper = new UiHelper(this);
    connect(ui->cbExpandContextView, SIGNAL(toggled(bool)), uiHelper, SLOT(expandContextView(bool)));
    connect(uiHelper, SIGNAL(contextViewCollapsed(bool)), ui->cbExpandContextView, SLOT(setChecked(bool)));
    connect(this, SIGNAL(selectionDialogClosed()), uiHelper, SLOT(selectionDialogClosed()));
    connect(this, SIGNAL(selectionDialogShown()), uiHelper, SLOT(selectionDialogShown()));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), uiHelper, SLOT(contextSelectionChanged(int)));
    /* errors, Application cursor and db related status bar messages */
    connect(snapDb, SIGNAL(queryExecuted(bool)), uiHelper, SLOT(dbDataRetrievalEnd(bool)));
    connect(snapDb, SIGNAL(querySent()), uiHelper, SLOT(dbDataRetrievalStart()));
    /* SNAP DB */
    /* snapDb->setConnectionParameters is called inside specific setup methods */

    /* update context view when context data ready */
    connect(snapDb, SIGNAL(contextListReady(const QList<ContextData>&)),
            ui->ctxView, SLOT(update(const QList<ContextData>&)));

    /* as soon as the database is available, obtain the list of contexts and the
     * list of the machine sections, ordered as in the section_order database table.
     */
    connect(snapDb, SIGNAL(databaseOpen()), snapDb, SLOT(getContextList()));
    connect(snapDb, SIGNAL(databaseOpen()), snapDb, SLOT(getOrderedSectionsList()));

    connect(snapDb, SIGNAL(dbConnectionInfo(QString)), statusBar(), SLOT(showMessage(QString)));

    connect(ui->actionReload_data_from_DB, SIGNAL(triggered()), snapDb, SLOT(getContextList()));

    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), snapDb, SLOT(getSnapshotList(int)));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), snapDb, SLOT(getAttInfoList(int)));

    /* PROGRESSBAR in status bar */
    ProgressBar * progressBar = new ProgressBar(statusBar());
    statusBar()->addWidget(progressBar, 2);

    /* LED in status bar
     */
    Led *led = new Led(statusBar()); /* would be reparented anyway to statusBar */
    connect(led, SIGNAL(clicked()), uiHelper, SLOT(ensureErrorViewVisible()));
    statusBar()->addPermanentWidget(led);

    /* Errors from database queries */
    connect(snapDb, SIGNAL(querySent()), led, SLOT(setMoving()));
    /* reset led when database is open */
    connect(snapDb, SIGNAL(databaseOpen()), led, SLOT(reset()));
    connect(snapDb, SIGNAL(queryExecuted(bool)), led, SLOT(setOk(bool)));
    connect(snapDb, SIGNAL(databaseError(QString,QString)),
            ui->errorView, SLOT(addError(QString,QString)));
    connect(snapDb, SIGNAL(databaseError(QString,QString)), led, SLOT(setError()));

    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), led, SLOT(setMoving()));
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), led, SLOT(forceOk()));


    /* RESTORE object
     */
    Restore *restore  = new Restore(this, mLogger);
    connect(ui->pbRestore, SIGNAL(clicked()), this, SLOT(restore()));
    connect(restore, SIGNAL(started()),  led, SLOT(setMoving())); /* LED */
    connect(restore, SIGNAL(started()),  ui->gbErrors, SLOT(hide())); /* HIDE ERRORS on restore click */
    connect(restore, SIGNAL(finished()), this, SLOT(restoreFinished()));
    connect(restore, SIGNAL(progress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(restore, SIGNAL(progress(int,int)), this, SLOT(progress(int,int)));
    connect(restore, SIGNAL(status(QString)), statusBar(), SLOT(showMessage(const QString&)));
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), restore, SLOT(clearErrors()));


    /* for time machine. It's not deleted, it's only hidden */
    connect(ui->pbDiffTimeMachine, SIGNAL(clicked()), this, SLOT(showDiffDialog()));

    /* DATABASE configuration dialog */
    connect(ui->actionDbConnection, SIGNAL(triggered()), this, SLOT(executeConfigDialog()));

    statusBar()->showMessage("Save Restore ready. Click on a context to begin.");

    /* FILTER on snapinfotreeview */
    connect(ui->pbHideFilter, SIGNAL(clicked()), uiHelper, SLOT(hideSnapViewFilter()));
    connect(ui->pbHideFilter, SIGNAL(clicked()), ui->leFilter, SLOT(clear()));
    connect(ui->pbClearFilter, SIGNAL(clicked()), ui->leFilter, SLOT(clear()));
    connect(ui->leFilter, SIGNAL(textChanged(QString)), uiHelper, SLOT(snapInfoViewFilterTextChanged(QString)));

    /* ABOUT dialogs */
    About *about = new About(this);
    connect(ui->actionAbout, SIGNAL(triggered()), about, SLOT(about()));
    connect(ui->actionAbout_THdb, SIGNAL(triggered()), about, SLOT(aboutTHdb()));
    connect(ui->actionAbout_QTango, SIGNAL(triggered()), about, SLOT(aboutQTango()));
    connect(ui->actionAbout_Qt, SIGNAL(triggered()), about, SLOT(aboutQt()));
    connect(ui->actionManual, SIGNAL(triggered()), about, SLOT(manual()));
    connect(ui->pbIKnow, SIGNAL(clicked()), uiHelper, SLOT(iKnowClicked()));

    /* ERROR tree view
     */
    connect(ui->errorView, SIGNAL(visible(bool)), ui->gbErrors, SLOT(setVisible(bool)));
    connect(ui->errorView, SIGNAL(visible(bool)), ui->gbErrors, SLOT(setChecked(bool)));
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), ui->errorView, SLOT(clear()));

    connect(ui->rbTimeMachine, SIGNAL(toggled(bool)), this, SLOT(setTimeMachineEnabled(bool)));
}

void SaveRestore::mSetupCommon2()
{
    AttributeDataTreeView *view = getAttributeDataTreeView();
    UiHelper *uiHelper = findChild<UiHelper *>();
    SnapDb *snapDb = findChild<SnapDb *>();
    qDebug() << __FUNCTION__ << "connecting snap db to view " << view;
    /* update attribute info when ready (data type, att name, format etc, not data values,
     * which are retrieved in different ways accordingly to operation mode, save restore or
     * hdb time machine).
     */
    connect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)), view, SLOT(update(const QList<AttInfo>&)));
    connect(snapDb, SIGNAL(orderedSectionsReady(QStringList)), view, SLOT(setSectionsList(QStringList)));
    connect(view, SIGNAL(error(QString,QString)), ui->errorView, SLOT(addError(QString,QString)));

    /* when the user clicks on the context list, he must wait until snap list and snap info
     * are loaded before selecting a snapshot.
     * Moreover, the items in the snapshot list and snap info view must be cleared.
     */
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), view, SLOT(clear()));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), view,  SLOT(setContextId(int)));

    SnapSelectionManager *snapSelectionManager = findChild<SnapSelectionManager *>();
    if(snapSelectionManager != NULL)
        printf("\e[1;31m***************** SNAP SELECTION MANAGER STILL AROUND in mSetupCommon2!!!!\n"
               "      ****************************\e[0m\n");
    snapSelectionManager = new SnapSelectionManager(view);
    printf("\e[1;31mCHECK SIGNAL SLOT CONNECTION FOR LOAD FROM ID SLOT!!!!\e[0m\n");
    connect(view, SIGNAL(snapInfoTreeViewSorted(int)), snapSelectionManager, SLOT(loadFromId(int)));
    connect(view, SIGNAL(clicked(const QModelIndex& )), snapSelectionManager, SLOT(itemClicked(const QModelIndex& )));
    /* Selection is now saved when item is clicked on the snapshotDataView, no more on ItemSelectionChanged.
     * So, it is necessary to trigger saveSelection if we want to programmatically save the selection
     * when not in response to a user's click on the tree view
     */
    connect(ui->pbIKnow, SIGNAL(clicked()), snapSelectionManager, SLOT(saveSelection()));
    connect(snapSelectionManager, SIGNAL(error(QString,QString)), ui->errorView, SLOT(addError(QString,QString)));
    connect(ui->pbIKnow, SIGNAL(clicked()), snapSelectionManager, SLOT(deselectReadOnlyAndNullItems()));

    /* SELECTION DIALOG */
    connect(view, SIGNAL(selectionRequest()), this, SLOT(executeSelectionDialog()));
    /* FILTER view */
    connect(view, SIGNAL(showFilterRequest()), uiHelper, SLOT(showSnapViewFilter()));
    connect(ui->leFilter, SIGNAL(textChanged(QString)), view, SLOT(filter(QString)));
    connect(ui->cbFilterOptions, SIGNAL(currentIndexChanged(int)), view, SLOT(setFilterMode(int)));
    /* invalid regular expression */
    connect(view, SIGNAL(filterRegexpValid(bool)), uiHelper, SLOT(setFilterLineEditValid(bool)));

    /* NULL and invalid items management */
    connect(view, SIGNAL(invalidItems(int,int)), uiHelper, SLOT(invalidItemsInSnap(int,int)));
    connect(view, SIGNAL(allItemsValid(bool)), uiHelper, SLOT(allItemsValidInSnap(bool)));

}

void SaveRestore::mDeleteSaveRestore()
{
    qDebug() << __FUNCTION__ << "- deleting widget " << ui->leftSplitter->widget(1) << "and "
                 << getAttributeDataTreeView();
    delete ui->leftSplitter->widget(1);
    delete this->getAttributeDataTreeView();
    /* snap info view db helper */
    delete findChild<SnapInfoViewDbHelper *>();
    delete findChild<Save *>();
}

void SaveRestore::mSetupSaveRestore()
{
    mDeleteTimeMachine();
    SnapDb *snapDb = findChild<SnapDb *>();
    Settings settings;
    bool isSnap = true;
    qDebug() << __FUNCTION__ << "Setting connection parameters to save restore";
    snapDb->setConnectionParameters(settings.dbConnectionParams(isSnap));
    Led *led = statusBar()->findChild<Led*>();
    UiHelper *uiHelper = findChild<UiHelper *>();
    uiHelper->init();
    ProgressBar *progressBar = statusBar()->findChild<ProgressBar *>();
    SnapshotListContainer *snapListContainer = new SnapshotListContainer(this);
    ui->leftSplitter->addWidget(snapListContainer);
   // ui->leftSplitter->setStretchFactor(0, 1);
   // ui->leftSplitter->setStretchFactor(1, 2);

    AttributeDataTreeView *attributeDV = new AttributeDataTreeView(this);
    QVBoxLayout *gbSnapDetailsLayout = qobject_cast<QVBoxLayout *>(ui->groupBoxSnapshotDetails->layout());
    gbSnapDetailsLayout->insertWidget(0, attributeDV, 2);
    attributeDV->init();

    connect(snapDb, SIGNAL(snapListReady(const QList<SnapData>&)),
            snapListContainer->snapshotView(), SLOT(update(const QList<SnapData>&)));
    connect(snapListContainer->snapshotView(), SIGNAL(snapSelectionChanged(int)),
            uiHelper, SLOT(snapSelectionChanged(int)));
    connect(snapListContainer->snapshotView(), SIGNAL(snapSelectionChanged(int)),
            attributeDV, SLOT(setSnapId(int)));
    /* DIFF push button and Diff Dialog. Hidden when in time machine */
    connect(snapListContainer->buttonDiff(), SIGNAL(clicked()), this, SLOT(showDiffDialog()));

    /* SnapInfoViewDbHelper is an intermediate that reads from the db the attribute values
     * corresponding to the attributes listed in the getAttributeDataTreeView().
     */
    SnapInfoViewDbHelper *snapInfoViewDbHelper = new SnapInfoViewDbHelper(attributeDV, snapDb);

    /* when the snapshot selection changes inside the snap view (bottom left), we ask the helper
     * to complete the partial information contained into getAttributeDataTreeView() (top right) with the
     * attribute values.
     */
    connect(snapListContainer->snapshotView(), SIGNAL(snapSelectionChanged(int)),
            snapInfoViewDbHelper, SLOT(getAttributeValuesForSnap(int)));

    /* snapInfoViewDbHelper simply maps the attValueListReady SIGNAL coming from SnapDb
     * into SnapInfoViewDbHelper's own attValueListReady SIGNAL. This is done in order
     * to let the helper do the full attribute value retrieval and final signal emission.
     */
    connect(snapInfoViewDbHelper, SIGNAL(attValueListReady(const QList<AttData>&, AttInfo::Type)),
            attributeDV, SLOT(update(const QList<AttData>&, AttInfo::Type)));
    connect(snapInfoViewDbHelper, SIGNAL(attValueListReady(const QList<AttData>&, AttInfo::Type)),
            uiHelper, SLOT(attributeValuesReady()));

    /* even though snapDb's query sent SIGNAL should be enough, the two following
     * connections make the led change to moving immediately. Signals coming from
     * snapDb are sent from a different thread. Qt::AutoConnection in THdbBasePrivate
     * connections would queue signal slots.
     */
    connect(snapListContainer->snapshotView(), SIGNAL(snapSelectionChanged(int)), led, SLOT(setMoving()));

    /* SAVE object
     */
    Save* save = new Save(this, snapDb, mLogger);
    save->setModel(attributeDV->model());
    connect(snapListContainer->buttonSave(), SIGNAL(clicked()), this, SLOT(save()));
    connect(save, SIGNAL(started()), led, SLOT(setMoving())); /* LED */
    connect(save, SIGNAL(started()), ui->gbErrors, SLOT(hide())); /* HIDE ERRORS ON SAVE click */
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), save, SLOT(setContextId(int)));
    /* error handling during save process */
    connect(save, SIGNAL(dbWriteError(QString,QString)), ui->errorView, SLOT(addError(QString,QString)));
    /* reload snapshot list after save process has completed, to refresh snapshot list */
    connect(save, SIGNAL(finished(int)), snapDb, SLOT(getSnapshotList(int)));
    connect(save, SIGNAL(finished(int, int)), this, SLOT(saveFinished(int, int)));
    /* progress and status string */
    connect(save, SIGNAL(readProgress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(save, SIGNAL(dbWriteProgress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(save, SIGNAL(status(QString)), statusBar(), SLOT(showMessage(const QString&)));
}

void SaveRestore::mDeleteTimeMachine()
{
    /* will delete TimeMachineAttributeDataTreeView */
    delete findChild<TimeMachineAttributeDataTreeView *>();
    delete findChild<Extractor *>();
    delete ui->leftSplitter->widget(1);
    ui->pbDiffTimeMachine->setVisible(false);
}

void SaveRestore::mSetupTimeMachine()
{
    mDeleteSaveRestore();

    SnapDb *snapDb = findChild<SnapDb *>();
    Settings settings;
    bool isSnap = false;
    snapDb->setConnectionParameters(settings.dbConnectionParams(isSnap));
    UiHelper *uiHelper = findChild<UiHelper *>();
    uiHelper->init();
    ui->pbDiffTimeMachine->setVisible(true);
    GroupBoxBackInTime *gbTime = new GroupBoxBackInTime(this);
    ui->leftSplitter->addWidget(gbTime);
    gbTime->setDisabled(true);

    TimeMachineAttributeDataTreeView *attributeDV = new TimeMachineAttributeDataTreeView(this);
    QVBoxLayout *gbSnapDetailsLayout = qobject_cast<QVBoxLayout *>(ui->groupBoxSnapshotDetails->layout());
    gbSnapDetailsLayout->insertWidget(0, attributeDV, 2);
    attributeDV->init();

    ProgressBar *progressBar = statusBar()->findChild<ProgressBar *>();
    Extractor * extractor = new Extractor(this);
    connect(extractor, SIGNAL(progress(int,int)), this, SLOT(extractionProgress(int,int)));
    connect(extractor, SIGNAL(progress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(extractor, SIGNAL(onError(QString,QString)),
            ui->errorView, SLOT(addError(QString,QString)));
    connect(extractor, SIGNAL(attValueListReady(const QList<AttData>& ,AttInfo::Type)),
            attributeDV, SLOT(update(const QList<AttData>& , AttInfo::Type)));

    qDebug() << __FUNCTION__ << QThread::currentThread();
    connect(gbTime->buttonFetchData(), SIGNAL(clicked()), this, SLOT(extractFromHdb()));
    connect(gbTime->buttonSelectFromDiary(), SIGNAL(clicked()), this, SLOT(executeSelectFromDiaryDialog()));
    gbTime->dateTimeEdit()->setDateTime(QDateTime::currentDateTime());
}

AttributeDataTreeView *SaveRestore::getAttributeDataTreeView() const
{
    return findChild<AttributeDataTreeView *>();
}

void SaveRestore::setTimeMachineEnabled(bool timeMachine)
{
    if(timeMachine && isTimeMachine())
        return;
    /* save the previously selected context id to restore the
     * snapshot information after switching mode on the AttributeDataTreeView
     */
    mSelectedContextId = ui->ctxView->selectedContextId();

    if(timeMachine)
        mSetupTimeMachine();
    else
        mSetupSaveRestore();
    mSetupCommon2();
}

bool SaveRestore::isTimeMachine() const
{
    return findChild<GroupBoxBackInTime *>() != NULL;
}

void SaveRestore::save()
{
    SnapshotListContainer *snapListContainer = findChild<SnapshotListContainer *>();
    findChild<Save *>()->save(snapListContainer->lineEditSaveComment()->text());
}

void SaveRestore::restore()
{
    findChild<Restore *>()->restore(getAttributeDataTreeView());
}

void SaveRestore::extractFromHdb()
{
    Extractor *extractor = findChild<Extractor *>();
    QPushButton *fetchDataBt = qobject_cast<QPushButton *>(sender());
    if(fetchDataBt && extractor->isRunning())
        extractor->cancel();
    else
    {
        getAttributeDataTreeView()->prepareForAttributeValuesQuery();
        QDateTime dt = findChild<GroupBoxBackInTime *>()->dateTimeEdit()->dateTime();
        QStringList attrs = getAttributeDataTreeView()->getAttributeList();
        extractor->getData(dt, attrs);
        findChild<UiHelper *>()->extractFromHdbStarted();
    }
}

void SaveRestore::progress(int s, int t)
{
    printf("\e[0;33m progress in SaveRestore %d/%d\n", s, t);
}

void SaveRestore::restoreProgress(int s, int t)
{
    printf("\e[0;33m progress in restore %d/%d\n", s, t);
}

void SaveRestore::extractionProgress(int s, int t)
{
    printf("\e[0;33m progress in TimeMachine %d/%d\n", s, t);
    if(s == t ) /* finished */
    {
        printf("\e[1;32mTimeMachine::progress: FINISHED, triggering selection on snapInfoView! ctx %d\e[0m\n",
               getAttributeDataTreeView()->currentContextId());
        findChild<SnapSelectionManager *>()->loadFromId(getAttributeDataTreeView()->currentContextId());
        findChild<UiHelper *>()->extractFromHdbEnded();
    }
}

void SaveRestore::saveFinished(int contextId, int newSnapId)
{
    Q_UNUSED(contextId);
    Q_UNUSED(newSnapId);
    Save* save = findChild<Save *>();
    if(save->readErrorCount() > 0)
    {
        QHash<QString, QString> attErrors = save->attErrors();
        foreach(QString key, attErrors.keys())
            ui->errorView->addError(key, attErrors.value(key));
        statusBar()->findChild<Led *>()->setError();
    }
    else
        statusBar()->findChild<Led *>()->setOk();
}

void SaveRestore::restoreFinished()
{
    Restore *restore = findChild<Restore *>();
    if(restore->writeErrorCount() > 0)
    {
        QHash<QString, QString> tangoErrorHash = restore->tangoErrors();
        foreach(QString key, tangoErrorHash.keys())
            ui->errorView->addError(key, tangoErrorHash.value(key));
        statusBar()->findChild<Led *>()->setError();
    }
    else
        statusBar()->findChild<Led *>()->setOk();
}

void SaveRestore::executeSelectionDialog()
{
    SelectionDialog *selectionDialog = new SelectionDialog(this, ui->ctxView->selectedContextId());
    connect(selectionDialog, SIGNAL(selectionChanged(const QStringList&)),
            getAttributeDataTreeView(), SLOT(selectByDevice(const QStringList &)));

    connect(selectionDialog, SIGNAL(finished(int)), this, SLOT(selectionDialogClosed(int)));

    Selection * selection = new Selection(this, findChild<SnapDb *>(),
                                          ui->ctxView->selectedContextId(),
                                          getAttributeDataTreeView()->selectionModel()->selection());

    connect(selection, SIGNAL(typesReady(QStringList)), selectionDialog,
            SLOT(addTypes(QStringList)));
    connect(selection, SIGNAL(deviceReady(QString,QString,QString)),
            selectionDialog, SLOT(addDevice(QString,QString,QString)));
    connect(selection, SIGNAL(dbDataRetrieved()), selectionDialog,
            SLOT(dataLoadedFromDb()));

    /* wait some time to avoid interface freezes */
    QTimer::singleShot(200, selection, SLOT(getSelection()));

    selectionDialog->show();

    emit selectionDialogShown();
}

void SaveRestore::selectionDialogClosed(int result)
{
    Selection * selection = findChild<Selection *>();
    if(result == QDialog::Rejected)
    {
        /* restore original selection, before dialog execution */
        getAttributeDataTreeView()->selectionModel()->clear();
        getAttributeDataTreeView()->selectionModel()->select(selection->savedItemSelection(), QItemSelectionModel::Select);
    }
    delete selection;
    emit selectionDialogClosed();
}

void SaveRestore::executeConfigDialog()
{
    SettingsDialog settingsDialog(this);
    settingsDialog.exec();
    qDebug() << __FUNCTION__ << "result settings dialog " << settingsDialog.result()
             << "accepted is " << QDialog::Accepted;
    if(settingsDialog.result() == QDialog::Accepted)
    {
        SnapDb *snapDb = findChild<SnapDb *>();
        snapDb->setConnectionParameters(Settings().dbConnectionParams(!isTimeMachine()));
        snapDb->getContextList();
        /* Extractor reloads configuration each time it is run in its thread (see Extractor::run)*/
    }
}

void SaveRestore::executeSelectFromDiaryDialog()
{
    Settings s;
    SelectFromDiaryDialog d(this, s.diarySource());
    d.setDiarySource(s.diarySource());
    int res = d.exec();
    if(res == QDialog::Accepted)
    {
        GroupBoxBackInTime *gbTime = findChild<GroupBoxBackInTime *>();
        if(gbTime)
        {
            printf("\e[1;34m setDateTime %s\e[0m\n", d.selectedDateTime().toString().toStdString().c_str());
            gbTime->dateTimeEdit()->setDateTime(d.selectedDateTime());
        }
    }
    else
        printf("SaveRestore.executeSelectFromDiaryDialog: cancelled\n");
}

void SaveRestore::showDiffDialog()
{
    if(!isTimeMachine())
    {
        SnapshotListContainer *slc = findChild<SnapshotListContainer *>();
        DiffDialog dlg(this, slc->snapshotView()->selectedSnapIds(), ui->ctxView->selectedContextId());
        SnapDb *snapDb = findChild<SnapDb *>();
        dlg.setSnapDb(snapDb);
        disconnect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)),
                   getAttributeDataTreeView(), SLOT(update(const QList<AttInfo>&)));
        dlg.getDiff();
        connect(&dlg, SIGNAL(selectSnapshot(int)), slc->snapshotView(), SLOT(select(int)));
        dlg.exec();
        connect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)),
                getAttributeDataTreeView(), SLOT(update(const QList<AttInfo>&)));
    }
}

SaveRestore::~SaveRestore()
{
    delete ui;
}
