#include "selectiontable.h"
#include "tableitemdelegate.h"
#include <macros.h>
#include <QtDebug>
#include <QHeaderView>
#include <QVariant>
#include <QPair>

SelectionTable::SelectionTable(QWidget *parent) :
    QTableWidget(parent)
{
    setItemDelegate(new TableItemDelegate(this));
    /* previously hover cell must be cleared */
    connect(itemDelegate(), SIGNAL(updateCell(QModelIndex)), this, SLOT(update(QModelIndex)));
    setSelectionMode(QAbstractItemView::MultiSelection);
    clear();
    setRowCount(0);
    setColumnCount(0);
    connect(this, SIGNAL(itemSelectionChanged()), this, SLOT(selectionChanged()));
    connect(this, SIGNAL(cellEntered(int,int)), this, SLOT(slotCellEntered(int, int)));
    setMouseTracking(true);
#if QT_VERSION >= 0x050000
  //  header()->setSectionResizeMode(QHeaderView::Interactive);
#else
  //  header()->setResizeMode(QHeaderView::Interactive);
#endif
}

void SelectionTable::selectionChanged()
{
    TableItemDelegate *tableItemDelegate = static_cast<TableItemDelegate *>(itemDelegate());
    QItemSelection selection = selectionModel()->selection();
    tableItemDelegate->clearSelection();
    for(int i = 0; i < selection.indexes().size(); i++)
    {
        tableItemDelegate->setSelected(selection.indexes().at(i));
    }
    emit deviceSelectionChanged(selectedDevices());
}

void SelectionTable::slotCellEntered(int row, int col)
{
    qDebug() << "devices" << m_idxAttnameHash.value(model()->index(row, col));
    static_cast<TableItemDelegate *>(itemDelegate())->setMouseIsOver(model()->index(row,col));
    emit deviceCellChanged(devicesForIndex(model()->index(row, col)));
}

void SelectionTable::addDevice(const QString& section,
                                  const QString& type,
                                  const QString& source)
{
    int row = addType(type);
    int column = addSection(section);
    if(row > -1 && column > -1)
    {
        m_addDeviceToIdxHash(model()->index(row, column), source);
    }
}

int SelectionTable::addSection(const QString& section)
{
    int column = -1;
    for(int i = 0; i < columnCount(); i++)
        if(horizontalHeaderItem(i)->text() == section)
            column = i;
    if(column == -1)
    {
        column = columnCount();
        setColumnCount(column + 1);
        setHorizontalHeaderItem(columnCount() - 1, new QTableWidgetItem(section));
    }
    return column;
}

int SelectionTable::addType(const QString& type)
{
    int row = -1;
    for(int i = 0; i < rowCount(); i++)
        if(verticalHeaderItem(i)->text() == type)
            row = i;
    if(row == -1)
    {
        row = rowCount();
        setRowCount(row + 1);
        setVerticalHeaderItem(rowCount() - 1, new QTableWidgetItem(type));
    }
    return row;
}

void SelectionTable::m_addDeviceToIdxHash(const QModelIndex& idx, const QString& attname)
{
    QStringList attList;
    if(m_idxAttnameHash.contains(idx))
        attList = m_idxAttnameHash.value(idx);
    if(!attList.contains(attname))
        attList << attname;
    m_idxAttnameHash[idx] = attList;
}

QStringList SelectionTable::devicesForIndex(const QModelIndex &idx) const
{
    QStringList ret;
    if(m_idxAttnameHash.contains(idx))
        ret = m_idxAttnameHash.value(idx);
    return ret;
}

QStringList SelectionTable::selectedDevices() const
{
    QStringList ret;
    TableItemDelegate *tableItemDelegate = static_cast<TableItemDelegate *>(itemDelegate());
    foreach(QModelIndex idx, tableItemDelegate->selectedIndexes())
    {
        if(m_idxAttnameHash.contains(idx))
        {
            ret << m_idxAttnameHash.value(idx);
        }
    }
    return ret;
}

QList<Cell> SelectionTable::selectedCells() const
{
    QList<Cell >  cells;
    foreach(QModelIndex idx, selectionModel()->selection().indexes())
    {
        Cell cell;
        cell.type = verticalHeaderItem(idx.row())->text();
        cell.section = horizontalHeaderItem(idx.column())->text();
        printf("\e[1;32madding cell %s %s\e[0m\n", qstoc(cell.type), qstoc(cell.section));
        cells << cell;
    }
    return cells;
}

void SelectionTable::select(const QList<Cell > & cells)
{
    QItemSelection itemSelection;
    foreach(Cell c, cells)
    {
        int row = verticalItemIdx(c.type);
        int column = horizontalItemIdx(c.section);
        printf("row %d col %d are selected %s %s\n", row, column, qstoc(c.section),
               qstoc(c.type));
        if(row > -1 && column > -1)
        {
            QModelIndex idx = model()->index(row, column);
            selectionModel()->select(idx, QItemSelectionModel::Select);
        }
    }
}

int SelectionTable::horizontalItemIdx(const QString& text) const
{
    for(int i = 0; i < columnCount(); i++)
        if(horizontalHeaderItem(i)->text() == text)
            return i;
    return -1;
}

int SelectionTable::verticalItemIdx(const QString& text) const
{
    for(int i = 0; i < rowCount(); i++)
        if(verticalHeaderItem(i)->text() == text)
            return i;
    return -1;
}

void SelectionTable::disableCellsWithoutDevices()
{
    for(int i = 0; i < rowCount(); i++)
    {
        for(int j = 0; j < columnCount(); j++)
        {
            QModelIndex idx = model()->index(i, j);
            if(!m_idxAttnameHash.contains(idx))
            {
                QTableWidgetItem *witem = item(i, j);
                if(!witem)
                {
                    witem = new QTableWidgetItem("");
                    setItem(i, j, witem);
                }
                witem->setFlags(witem->flags() & !Qt::ItemIsEnabled);
                static_cast<TableItemDelegate *>(itemDelegate())->setDisabled(idx);
            }
        }
    }
}
