#include "snapshotview.h"
#include "database/snapdata.h"
#include "modelview/itemmodel.h"
#include <QHeaderView>
#include <QVariant>
#include <macros.h>
#include <QtDebug>

SnapshotView::SnapshotView(QWidget *parent) : QTreeView(parent)
{
    setRootIsDecorated(false);

#if QT_VERSION >= 0x050000
    header()->setSectionResizeMode(QHeaderView::Interactive);
#else
    header()->setResizeMode(QHeaderView::Interactive);
#endif
    QList<QVariant> headerData;
    headerData << "Id" << "Time" << "Comment" << "Id Context";
    ItemModel *itemModel = new ItemModel(this, headerData);
    setModel(itemModel);

    setSelectionMode(QAbstractItemView::ExtendedSelection);

    /* manage context selection */
    connect(selectionModel(), SIGNAL(currentChanged(QModelIndex,QModelIndex)),
            this, SLOT(itemSelectionChanged(QModelIndex,QModelIndex)));
}

void SnapshotView::itemSelectionChanged(const QModelIndex& current, const QModelIndex &previous)
{
    Q_UNUSED(previous);
    qDebug() << __FUNCTION__ << "current " << current << "prev " << previous;
    if(current.isValid())
    {
        int row = current.row();
        int snapId = model()->data(model()->index(row, ColumnSnapId)).toInt();
        if(current != QModelIndex())
            emit snapSelectionChanged(snapId);
    }
}

void SnapshotView::update(const QList<SnapData>& snaplist)
{
    printf("updating snapshot list\n");
    int snapIdMax = -1, ithSnapId;
    int previousCtxId = -1;
    /* retrieve the maximum snapshot id so that if the snap list for the
     * _SAME_ context is reloaded, we can easily select the last snap id.
     */
    for(int i = 0; i < model()->rowCount(); i++)
    {
        ithSnapId = model()->data(model()->index(i, ColumnSnapId), Qt::DisplayRole).toInt();
        if(ithSnapId > snapIdMax)
            snapIdMax = ithSnapId;
        if(previousCtxId == -1)
            previousCtxId = model()->data(model()->index(i, ColumnContextId), Qt::DisplayRole).toInt();
    }
    /* clear view */
    model()->removeRows(0, model()->rowCount());
    ItemModel *itemModel = static_cast<ItemModel *>(model());
    foreach(SnapData sd, snaplist)
    {
        /* final false parameter not to emit dataChanged at every setData */
        itemModel->insertRows(model()->rowCount(), 1);
        itemModel->setData(model()->index(model()->rowCount() - 1, ColumnSnapId), sd.idSnap, Qt::DisplayRole, false);
        itemModel->setData(model()->index(model()->rowCount() - 1, ColumnTime), sd.time, Qt::DisplayRole, false);
        itemModel->setData(model()->index(model()->rowCount() - 1, ColumnComment), sd.comment, Qt::DisplayRole, false);
        itemModel->setData(model()->index(model()->rowCount() - 1, ColumnContextId), sd.idContext, Qt::DisplayRole, false);

        /* when reloading the view, select the last inserted snapshot row, checking
         * that 1. we are referring to the same context; 2. there effectively is a new snap
         * (which has greatest id, by definition)
         */
        if(previousCtxId == sd.idContext && sd.idSnap > snapIdMax)
        {
            selectionModel()->select(QItemSelection(itemModel->index(model()->rowCount() - 1, 0),
                                                    itemModel->index(model()->rowCount() - 1, itemModel->columnCount() -1)), QItemSelectionModel::Select);
            emit snapSelectionChanged(sd.idSnap);
        }
    }
    /* trigger an update from the top left to the bottom right index */
    itemModel->update(itemModel->index(0, 0), itemModel->index(itemModel->rowCount() - 1, 3));
    header()->resizeSections(QHeaderView::ResizeToContents);
    if(model()->rowCount() == snaplist.size())
        scrollTo(model()->index(model()->rowCount() - 1, 0));

    setSelectionMode(QAbstractItemView::ExtendedSelection);
}

QList<int> SnapshotView::selectedSnapIds() const
{
    QList<int> snapids;
    QItemSelection itemSelection = selectionModel()->selection();
    QModelIndexList idxList = itemSelection.indexes();
    for(int i = 0; i < idxList.size(); i++)
    {
        /* select only one index per row ;-) */
        if(idxList.at(i).column() == ColumnSnapId)
            snapids << model()->data(idxList.at(i), Qt::DisplayRole).toInt();
    }
    return snapids;
}

void SnapshotView::select(int snapId)
{
    int row = -1;
    selectionModel()->clearSelection();
    for(int i = 0; i < model()->rowCount(); i++)
    {
        if(model()->data(model()->index(i, ColumnSnapId), Qt::DisplayRole).toInt() == snapId)
        {
            row = i;
            break;
        }
    }
    QItemSelection itemSelection(model()->index(row, 0),
                                 model()->index(row, model()->columnCount() -1));
    selectionModel()->select(itemSelection, QItemSelectionModel::Select);
    emit snapSelectionChanged(snapId);
}
