#include "saverestore.h"
#include "snapinfoviewdbhelper.h"
#include "ui_saverestore.h"
#include "database/snapdb.h"
#include "database/attvalue.h"
#include "database/attinfoquery.h"
#include "save/save.h"
#include "restore/snapselectionmanager.h"
#include "led.h"
#include "about.h"
#include "progressbar.h"
#include "uihelper.h"
#include "restore/restore.h"
#include "diff/diffdialog.h"
#include "selection/selectiondialog.h"
#include "selection/selection.h"
#include "settings/settingsdialog.h"
#include "settings/settings.h"
#include "log/logger.h"
#include <QMetaType>
#include <QTimer>
#include <action_factory.h>


SaveRestore::SaveRestore(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::SaveRestore)
{
    /* act on ActionFactory to disable TLog/TUtil::addLog for all sources targets that
     * will be created by SaveRestore.
     */
#if ((VER_MAJ >3 && VER_MIN > 1 && VER_FIX > 2) || (VER_MAJ >3 && VER_MIN > 2))
    ActionFactory::actionFactory()->setLoggingDisabled(true);
#endif

    ActionFactory::actionFactory()->setDefaultRefreshMode(POLLED_REFRESH);

    ui->setupUi(this);
    ui->gbErrors->setHidden(true);

    /* log directory from command line arguments */
    QString logDir = "";
    if(qApp->arguments().contains("--log-dir") && qApp->argc() > qApp->arguments().indexOf("--log-dir") + 1)
    {
        logDir = qApp->arguments().at(qApp->arguments().indexOf("--log-dir") + 1);
    }
    Logger *logger = new Logger(logDir);

    /* UI HELPER: manages interface interaction with user, such as
     * shows/hides enables/disables part of the user interface
     * according to determined user actions.
     */
    UiHelper *uiHelper = new UiHelper(this);
    connect(ui->cbExpandContextView, SIGNAL(toggled(bool)), uiHelper, SLOT(expandContextView(bool)));
    connect(uiHelper, SIGNAL(contextViewCollapsed(bool)), ui->cbExpandContextView, SLOT(setChecked(bool)));
    connect(ui->cbOptimizeSnapView, SIGNAL(toggled(bool)), uiHelper, SLOT(optimizeSnapshotView(bool)));
    ui->cbExpandContextView->setChecked(true);
    connect(this, SIGNAL(selectionDialogClosed()), uiHelper, SLOT(selectionDialogClosed()));
    connect(this, SIGNAL(selectionDialogShown()), uiHelper, SLOT(selectionDialogShown()));

    /* SNAP DB */
    Settings settings;
    SnapDb *snapDb = new SnapDb(this);
    snapDb->setConnectionParameters(settings.dbConnectionParams());

    connect(snapDb, SIGNAL(contextListReady(const QList<ContextData>&)),
            ui->ctxView, SLOT(update(const QList<ContextData>&)));

    connect(snapDb, SIGNAL(snapListReady(const QList<SnapData>&)),
            ui->snapView, SLOT(update(const QList<SnapData>&)));

    connect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)),
            ui->snapInfoView, SLOT(update(const QList<AttInfo>&)));

    connect(snapDb, SIGNAL(orderedSectionsReady(QStringList)), ui->snapInfoView,
            SLOT(setSectionsList(QStringList)));

    /* as soon as the database is available, obtain the list of contexts and the
     * list of the machine sections, ordered as in the section_order database table.
     */
    connect(snapDb, SIGNAL(databaseOpen()), snapDb, SLOT(getContextList()));
    connect(snapDb, SIGNAL(databaseOpen()), snapDb, SLOT(getOrderedSectionsList()));

    connect(snapDb, SIGNAL(dbConnectionInfo(QString)), statusBar(), SLOT(showMessage(QString)));

    connect(ui->actionReload_data_from_DB, SIGNAL(triggered()), snapDb, SLOT(getContextList()));

    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), snapDb, SLOT(getSnapshotList(int)));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), snapDb, SLOT(getAttInfoList(int)));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), uiHelper, SLOT(contextSelectionChanged(int)));
    connect(ui->snapView, SIGNAL(snapSelectionChanged(int)), uiHelper, SLOT(snapSelectionChanged(int)));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), ui->snapInfoView,  SLOT(setContextId(int)));
    connect(ui->snapView, SIGNAL(snapSelectionChanged(int)), ui->snapInfoView, SLOT(setSnapId(int)));
    connect(ui->snapInfoView, SIGNAL(error(QString,QString)), ui->errorView, SLOT(addError(QString,QString)));

    /* when the user clicks on the context list, he must wait until snap list and snap info
     * are loaded before selecting a snapshot.
     * Moreover, the items in the snapshot list and snap info view must be cleared.
     */
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), ui->snapInfoView, SLOT(clear()));

    /* SnapInfoViewDbHelper is an intermediate that reads from the db the attribute values
     * corresponding to the attributes listed in the ui->snapInfoView.
     */
    SnapInfoViewDbHelper *snapInfoViewDbHelper = new SnapInfoViewDbHelper(ui->snapInfoView, snapDb);

    /* when the snapshot selection changes inside the snap view (bottom left), we ask the helper
     * to complete the partial information contained into ui->snapInfoView (top right) with the
     * attribute values.
     */
    connect(ui->snapView, SIGNAL(snapSelectionChanged(int)), snapInfoViewDbHelper, SLOT(getAttributeValuesForSnap(int)));

    /* snapInfoViewDbHelper simply maps the attValueListReady SIGNAL coming from SnapDb
     * into SnapInfoViewDbHelper's own attValueListReady SIGNAL. This is done in order
     * to let the helper do the full attribute value retrieval and final signal emission.
     */
    connect(snapInfoViewDbHelper, SIGNAL(attValueListReady(const QList<AttValue>&, AttInfo::Type)),
            ui->snapInfoView, SLOT(update(const QList<AttValue>&, AttInfo::Type)));

    /* PROGRESSBAR in status bar */
    ProgressBar * progressBar = new ProgressBar(statusBar());
    statusBar()->addWidget(progressBar, 2);

    /* LED in status bar
     */
    Led *led = new Led(statusBar()); /* would be reparented anyway to statusBar */
    connect(led, SIGNAL(clicked()), uiHelper, SLOT(ensureErrorViewVisible()));
    statusBar()->addPermanentWidget(led);

    /* Errors from database queries */
    connect(snapDb, SIGNAL(querySent()), led, SLOT(setMoving()));
    /* reset led when database is open */
    connect(snapDb, SIGNAL(databaseOpen()), led, SLOT(reset()));
    connect(snapDb, SIGNAL(queryExecuted(bool)), led, SLOT(setOk(bool)));
    connect(snapDb, SIGNAL(databaseError(QString,QString)),
            ui->errorView, SLOT(addError(QString,QString)));
    connect(snapDb, SIGNAL(databaseError(QString,QString)), led, SLOT(setError()));

    /* errors, Application cursor and db related status bar messages */
    connect(snapDb, SIGNAL(queryExecuted(bool)), uiHelper, SLOT(dbDataRetrievalEnd(bool)));
    connect(snapDb, SIGNAL(querySent()), uiHelper, SLOT(dbDataRetrievalStart()));

    /* event though snapDb's query sent SIGNAL should be enough, the two following
     * connections make the led change to moving immediately. Signals coming from
     * snapDb are sent from a different thread. Qt::AutoConnection in THdbBasePrivate
     * connections would queue signal slots.
     */
    connect(ui->snapView, SIGNAL(snapSelectionChanged(int)), led, SLOT(setMoving()));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), led, SLOT(setMoving()));
    
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), led, SLOT(forceOk()));

    /* SAVE object
     */
    Save* save = new Save(this, snapDb, logger);
    save->setModel(ui->snapInfoView->model());
    connect(ui->pbSave, SIGNAL(clicked()), this, SLOT(save()));
    connect(save, SIGNAL(started()), led, SLOT(setMoving())); /* LED */
    connect(save, SIGNAL(started()), ui->gbErrors, SLOT(hide())); /* HIDE ERRORS ON SAVE click */
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), save, SLOT(setContextId(int)));
    /* error handling during save process */
    connect(save, SIGNAL(dbWriteError(QString,QString)), ui->errorView, SLOT(addError(QString,QString)));
    /* reload snapshot list after save process has completed, to refresh snapshot list */
    connect(save, SIGNAL(finished(int)), snapDb, SLOT(getSnapshotList(int)));
    connect(save, SIGNAL(finished(int, int)), this, SLOT(saveFinished(int, int)));
    /* progress and status string */
    connect(save, SIGNAL(readProgress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(save, SIGNAL(dbWriteProgress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(save, SIGNAL(status(QString)), statusBar(), SLOT(showMessage(const QString&)));

    /* ERROR tree view
     */
    connect(ui->errorView, SIGNAL(visible(bool)), ui->gbErrors, SLOT(setVisible(bool)));
    connect(ui->errorView, SIGNAL(visible(bool)), ui->gbErrors, SLOT(setChecked(bool)));
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), ui->errorView, SLOT(clear()));

    /* RESTORE object
     */
    Restore *restore  = new Restore(this, logger);
    connect(ui->pbRestore, SIGNAL(clicked()), this, SLOT(restore()));
    connect(restore, SIGNAL(started()),  led, SLOT(setMoving())); /* LED */
    connect(restore, SIGNAL(started()),  ui->gbErrors, SLOT(hide())); /* HIDE ERRORS on restore click */
    connect(restore, SIGNAL(finished()), this, SLOT(restoreFinished()));
    connect(restore, SIGNAL(progress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(restore, SIGNAL(progress(int,int)), this, SLOT(progress(int,int)));
    connect(restore, SIGNAL(status(QString)), statusBar(), SLOT(showMessage(const QString&)));
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), restore, SLOT(clearErrors()));

    /* RESTORE selection manager */
    SnapSelectionManager *snapSelectionManager = new SnapSelectionManager(ui->snapInfoView);
    connect(ui->snapView, SIGNAL(snapSelectionChanged(int)), snapSelectionManager,
            SLOT(load(int)));

    connect(ui->snapInfoView, SIGNAL(snapInfoTreeViewSorted(int)), snapSelectionManager,
            SLOT(load(int)));

    connect(ui->snapInfoView->selectionModel(), SIGNAL(selectionChanged(QItemSelection,QItemSelection)),
            snapSelectionManager, SLOT(itemSelectionChanged(QItemSelection,QItemSelection)));
    /* selection manager error handling. They are not fatal, so it is enough to
     * signal them to the user (simply, the selection is not remembered.
     */
    connect(snapSelectionManager, SIGNAL(error(QString,QString)), ui->errorView,
            SLOT(addError(QString,QString)));

    /* DIFF push button and Diff Dialog */
    connect(ui->pbDiff, SIGNAL(clicked()), this, SLOT(showDiffDialog()));

    /* SELECTION DIALOG */
    connect(ui->snapInfoView, SIGNAL(selectionRequest()), this, SLOT(executeSelectionDialog()));
    /* FILTER view */
    connect(ui->snapInfoView, SIGNAL(showFilterRequest()), uiHelper, SLOT(showSnapViewFilter()));

    /* DATABASE configuration dialog */
    connect(ui->actionDbConnection, SIGNAL(triggered()), this, SLOT(executeConfigDialog()));

    statusBar()->showMessage("Save Restore ready. Click on a context to begin.");

    /* FILTER on snapinfotreeview */
    connect(ui->pbHideFilter, SIGNAL(clicked()), uiHelper, SLOT(hideSnapViewFilter()));
    connect(ui->pbHideFilter, SIGNAL(clicked()), ui->leFilter, SLOT(clear()));
    connect(ui->pbClearFilter, SIGNAL(clicked()), ui->leFilter, SLOT(clear()));
    connect(ui->leFilter, SIGNAL(textChanged(QString)), ui->snapInfoView, SLOT(filter(QString)));
    connect(ui->cbFilterOptions, SIGNAL(currentIndexChanged(int)), ui->snapInfoView, SLOT(setFilterMode(int)));
    connect(ui->leFilter, SIGNAL(textChanged(QString)), uiHelper, SLOT(snapInfoViewFilterTextChanged(QString)));
    /* invalid regular expression */
    connect(ui->snapInfoView, SIGNAL(filterRegexpValid(bool)), uiHelper, SLOT(setFilterLineEditValid(bool)));

    /* ABOUT dialogs */
    About *about = new About(this);
    connect(ui->actionAbout, SIGNAL(triggered()), about, SLOT(about()));
    connect(ui->actionAbout_THdb, SIGNAL(triggered()), about, SLOT(aboutTHdb()));
    connect(ui->actionAbout_QTango, SIGNAL(triggered()), about, SLOT(aboutQTango()));
    connect(ui->actionAbout_Qt, SIGNAL(triggered()), about, SLOT(aboutQt()));
    connect(ui->actionManual, SIGNAL(triggered()), about, SLOT(manual()));

    /* NULL and invalid items management */
    connect(ui->snapInfoView, SIGNAL(invalidItems(int,int)), uiHelper, SLOT(invalidItemsInSnap(int,int)));
    connect(ui->snapInfoView, SIGNAL(allItemsValid(bool)), uiHelper, SLOT(allItemsValidInSnap(bool)));
    connect(ui->pbIKnow, SIGNAL(clicked()), uiHelper, SLOT(iKnowClicked()));
    connect(ui->pbIKnow, SIGNAL(clicked()), snapSelectionManager, SLOT(deselectReadOnlyAndNullItems()));
}

void SaveRestore::save()
{
    findChild<Save *>()->save(ui->leSaveComment->text());
}

void SaveRestore::restore()
{
    findChild<Restore *>()->restore(ui->snapInfoView);
}

void SaveRestore::progress(int s, int t)
{
//    printf("\e[0;33m progress in SaveRestore %d/%d\n", s, t);
}

void SaveRestore::saveFinished(int contextId, int newSnapId)
{
    Q_UNUSED(contextId);
    Q_UNUSED(newSnapId);
    Save* save = findChild<Save *>();
    if(save->readErrorCount() > 0)
    {
        QHash<QString, QString> attErrors = save->attErrors();
        foreach(QString key, attErrors.keys())
            ui->errorView->addError(key, attErrors.value(key));
        statusBar()->findChild<Led *>()->setError();
    }
    else
        statusBar()->findChild<Led *>()->setOk();
}

void SaveRestore::restoreFinished()
{
    Restore *restore = findChild<Restore *>();
    if(restore->writeErrorCount() > 0)
    {
       QHash<QString, QString> tangoErrorHash = restore->tangoErrors();
       foreach(QString key, tangoErrorHash.keys())
          ui->errorView->addError(key, tangoErrorHash.value(key));
       statusBar()->findChild<Led *>()->setError();
    }
    else
        statusBar()->findChild<Led *>()->setOk();
}

void SaveRestore::executeSelectionDialog()
{
    SelectionDialog *selectionDialog = new SelectionDialog(this, ui->ctxView->selectedContextId());
    connect(selectionDialog, SIGNAL(selectionChanged(const QStringList&)),
            ui->snapInfoView, SLOT(selectByDevice(const QStringList &)));

    connect(selectionDialog, SIGNAL(finished(int)), this, SLOT(selectionDialogClosed(int)));

    Selection * selection = new Selection(this, findChild<SnapDb *>(),
              ui->ctxView->selectedContextId(),
              ui->snapInfoView->selectionModel()->selection());

    connect(selection, SIGNAL(typesReady(QStringList)), selectionDialog,
            SLOT(addTypes(QStringList)));
    connect(selection, SIGNAL(deviceReady(QString,QString,QString)),
            selectionDialog, SLOT(addDevice(QString,QString,QString)));
    connect(selection, SIGNAL(dbDataRetrieved()), selectionDialog,
            SLOT(dataLoadedFromDb()));

    /* wait some time to avoid interface freezes */
    QTimer::singleShot(200, selection, SLOT(getSelection()));

    selectionDialog->show();

    emit selectionDialogShown();
}

void SaveRestore::selectionDialogClosed(int result)
{
    Selection * selection = findChild<Selection *>();
    if(result == QDialog::Rejected)
    {
        /* restore original selection, before dialog execution */
        ui->snapInfoView->selectionModel()->clear();
        ui->snapInfoView->selectionModel()->select(selection->savedItemSelection(), QItemSelectionModel::Select);
    }
    delete selection;
    emit selectionDialogClosed();
}

void SaveRestore::executeConfigDialog()
{
    SettingsDialog settingsDialog(this, findChild<SnapDb *>());
    settingsDialog.exec();
}

void SaveRestore::showDiffDialog()
{
    DiffDialog dlg(this, ui->snapView->selectedSnapIds(), ui->ctxView->selectedContextId());
    SnapDb *snapDb = findChild<SnapDb *>();
    dlg.setSnapDb(snapDb);
    disconnect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)),
            ui->snapInfoView, SLOT(update(const QList<AttInfo>&)));
    dlg.getDiff();
    connect(&dlg, SIGNAL(selectSnapshot(int)), ui->snapView, SLOT(select(int)));
    dlg.exec();
    connect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)),
      ui->snapInfoView, SLOT(update(const QList<AttInfo>&)));
}

SaveRestore::~SaveRestore()
{
    delete ui;
}
