#include "extractor.h"
#include <hdbxsettings.h>
#include <configurationparser.h>
#include <macros.h>
#include <datetimeutils.h> /* hdb extractor */
#include "../settings/settings.h"

#include <QMessageBox>
#include <QStringList>
#include <QApplication>
#include <QtDebug>

#include "extractorprivate.h"
#include "xevents.h"

Extractor::~Extractor()
{
    qDebug() << __FUNCTION__ << "isFinished" << isFinished();
    wait();
    delete d_ptr; /* deletes d_ptr->mHdbExtractor and d_ptr->mHdbXSettings */
}

Extractor::Extractor(QObject *parent) : QThread(parent)
{
    d_ptr = new ExtractorPrivate();
    d_ptr->mTotSteps = 5;
    d_ptr->mStepSize = 20;
    d_ptr->mHdbExtractor = NULL;
    setObjectName("ExtractorThread");
    connect(this, SIGNAL(finished()), this, SLOT(threadFinished()));
}

void Extractor::mSetParamsFromHdbXSettings(HdbXSettings *q)
{
    d_ptr->mHost = QString::fromStdString(q->get("dbhost"));
    d_ptr->mUser = QString::fromStdString(q->get("dbuser"));
    d_ptr->mPass = QString::fromStdString(q->get("dbpass"));
    d_ptr->mPort = QString::fromStdString(q->get("dbport")).toUInt();
    d_ptr->mDb   = QString::fromStdString(q->get("dbname"));

    q->set("FillFromThePastMode", "WidenWindow");

    d_ptr->mDbType = Hdbextractor::DBUNDEFINED;
    if(d_ptr->mDb == "hdb")
        d_ptr->mDbType = Hdbextractor::HDBMYSQL;
    else if(d_ptr->mDb == "hdbpp")
        d_ptr->mDbType = Hdbextractor::HDBPPMYSQL;
    else
        d_ptr->mErrors.append("Available databases: \"hdb\", \"hdbpp\"");
}

void Extractor::run()
{
    char start_time[64], stop_time[64];
    d_ptr->mStep = 0;
    d_ptr->mCancel = false;

    if(d_ptr->mHdbExtractor)
    {
        printf("\e[1;35mExtractor::run: deleting hdb extractor instance %p\e[0m\n", d_ptr->mHdbExtractor);
        delete d_ptr->mHdbExtractor;
    }
    HdbXSettings *HdbXSettings = Settings().getHdbXSettings();
    d_ptr->mHdbExtractor  = new Hdbextractor(this);
    mSetParamsFromHdbXSettings(HdbXSettings);
    /* Settings.getHdbXSettings allocates a new HdbXSettings object.
     * The responsibility of that object is immediately passed to the Hdbextractor which
     * will take care of its lifetime.
     */
    d_ptr->mHdbExtractor->setHdbXSettings(HdbXSettings);
    /* after connection parameters setup, try to connect */
    d_ptr->mHdbExtractor->connect(d_ptr->mDbType, qstoc(d_ptr->mHost), qstoc(d_ptr->mDb),
                                  qstoc(d_ptr->mUser), qstoc(d_ptr->mPass), d_ptr->mPort);
    strncpy(start_time, d_ptr->mStartDateTime.toString("yyyy-MM-dd hh:mm:ss").toStdString().c_str(), 64);
    strncpy(stop_time, d_ptr->mStopDateTime.toString("yyyy-MM-dd hh:mm:ss").toStdString().c_str(), 64);

    qDebug() << __FUNCTION__ << "Thread" << QThread::currentThread() << d_ptr->mStartDateTime << start_time
             << d_ptr->mStartDateTime.toString("yyyy-MM-dd hh:mm:ss").toStdString().c_str();

    /* fetch data */
    for( int i = 0; i < d_ptr->mAttributeList.size(); i++)
    {
        QString a = d_ptr->mAttributeList.at(i);
        bool success = d_ptr->mHdbExtractor->getData(qstoc(a), start_time, stop_time);
        if(!success)
            d_ptr->mErrors << d_ptr->mHdbExtractor->getErrorMessage();
        if(d_ptr->mCancel)
        {
            d_ptr->mErrors << QString("Canceled by the user at step %1").arg(i + 1);
            break;
        }
    }

    d_ptr->mHdbExtractor->disconnect();
}

void Extractor::getData(const QDateTime &dateTime, const QStringList &attributes)
{
    d_ptr->mStartDateTime = dateTime;
    d_ptr->mStopDateTime = dateTime;
    qDebug() << __FUNCTION__ << "Thread" << QThread::currentThread() << d_ptr->mStartDateTime << dateTime;
    d_ptr->mAttributeList = attributes;
    start();
}

void Extractor::getData(const QDateTime &startDt, const QDateTime &stopDt, const QStringList &attributes)
{
    d_ptr->mStartDateTime = startDt;
    d_ptr->mStopDateTime = stopDt;
    d_ptr->mAttributeList = attributes;
    start();
}

void Extractor::setProgressUpdateGranularity(int steps)
{
    d_ptr->mTotSteps = steps;
    d_ptr->mStepSize = qRound(100.0 / d_ptr->mTotSteps);
}

bool Extractor::event(QEvent *e)
{
    if(e->type() == QEvent::User + 1) /* NewXDataEvent */
    {
        NewXDataEvent *nxe = static_cast<NewXDataEvent *>(e);
        emit attValueListReady(nxe->data, nxe->type);
        //   delete nxe;
        return true;
    }
    else if(e->type() == QEvent::User + 2) /* error */
    {
        XErrorEvent *xe = static_cast<XErrorEvent *>(e);
        foreach(QString err, xe->errors)
            emit onError("Historical Database Extractor", err);
        //   delete xe;
        return true;
    }
    else if(e->type() == QEvent::User + 3) /* progress */
    {
        ProgressEvent *pe = static_cast<ProgressEvent *>(e);
        emit progress(pe->step, pe->total);
        //   delete xe;
        return true;
    }
    else
        return QThread::event(e);
}


void Extractor::onSourceProgressUpdate(const char * source, double percent)
{
    Q_UNUSED(source);
    Q_UNUSED(percent);
}


void Extractor::onExtractionFinished(int, double)
{

}

void Extractor::onSourceExtractionFinished(const char *source , int totalRows, double elapsed )
{
    qDebug() << __FUNCTION__ << "Thread" <<  QThread::currentThread()
             << source << elapsed << totalRows;

    d_ptr->mStep++;
    /* update progress in 5 steps */
    const int totSources = d_ptr->mAttributeList.size();
    const int progressStep = qRound(totSources * d_ptr->mStepSize / 100.0);
    int currentProgress = 0;
    if(progressStep > 0)
        currentProgress = d_ptr->mStep / progressStep;


    qDebug() << __FUNCTION__ << "currentprogress" << currentProgress
             << "d_ptr->mProgress" << d_ptr->mProgress
             << "mStep" << d_ptr->mStep << " toatl step " << totSources;

    /* get data and post results (via attValueListReady signal) if progress step advances
     * or if this is the last extracted source (d_ptr->mStep == totSources)
     */
    if(currentProgress != d_ptr->mProgress || d_ptr->mStep == totSources)
    {
        /* empty XVariant data queue and send results to the view */
        bool conversionOk;
        d_ptr->mProgress = currentProgress;
        emit progress(currentProgress, d_ptr->mTotSteps);
        XVariant::DataType dt;
        XVariant::DataFormat f;
        XVariant::Writable w;
        /* empty data buffer and emit new data */
        std::vector<XVariant> data;
        /* make just one copy */
        d_ptr->mHdbExtractor->get(data);
        struct timeval tv;

        printf("\e[1;34m data size from extractor %d\e[0m\n", data.size());
        for(size_t i = 0; i < data.size(); i++)
        {

            XVariant &v = data[i];
            if(!v.isValid())
            {
                d_ptr->mErrors.append(QString("Extractor::onFinished: XVariant \"%1\" is null").arg(v.getSource()));
            }
            else
            {
                AttData val(v.getSource(), "NULL", "NULL");
                tv = v.getTimevalTimestamp();
                time_t secsSinceEpoch = tv.tv_sec; /* 64 bit?? */
                val.dateTime = QDateTime::fromTime_t(secsSinceEpoch);
                f = v.getFormat();
                dt = v.getType();
                w = v.getWritable();
                if(w == XVariant::RO)
                    val.writable = Tango::READ;
                else if(w == XVariant::RW)
                    val.writable = Tango::READ_WRITE;
                else if(w == XVariant::WO)
                    val.writable = Tango::WRITE;

                if(w == XVariant::RO || w == XVariant::RW)
                {
                    val.value = QString::fromStdString(v.convertToString(true, &conversionOk));
                    if(!conversionOk)
                        d_ptr->mErrors.append(QString("Extractor::onFinished: conversion error in \"%1\"")
                                              .arg(v.getSource()));
                    if(!conversionOk || val.value.isEmpty())
                        val.value = "NULL";
                }
                if(w == XVariant::WO || w == XVariant::RW)
                {
                    val.setpoint = QString::fromStdString(v.convertToString(false, &conversionOk));
                    if(!conversionOk)
                        d_ptr->mErrors.append(QString("Extractor::onFinished: conversion error in \"%1\"")
                                              .arg(v.getSource()));
                    if(!conversionOk || val.setpoint.isEmpty())
                        val.setpoint = "NULL";
                    printf("\e[1;33msetpoint %s\e[0m\n", qstoc(val.setpoint));
                }

                AttInfo::Type at = mGetAttInfoType(f, dt, w);
                val.type = at;
                if(d_ptr->mAttValueHash.contains(at))
                {
                    /* get a reference to the existing list */
                    QList<AttData> &currentValueList = d_ptr->mAttValueHash[at];
                    currentValueList.append(val); /* and append new val */
                }
                else /* create new list for that AttInfo type */
                {
                    d_ptr->mAttValueHash.insert(at, QList<AttData> () << val);
                }
            }
        } /* for i < data.size() */

        foreach(AttInfo::Type type, d_ptr->mAttValueHash.keys())
        {
            /* copy into valueList the value list */
            NewXDataEvent *xdatae = new NewXDataEvent(d_ptr->mAttValueHash.value(type), type);
            qApp->postEvent(this, xdatae);
            qDebug() << __FUNCTION__ << QThread::currentThread() << " --> POST EVENT <-- ";

            /* let progress be 100% when thread finishes */
            if(currentProgress < d_ptr->mTotSteps)
                emit progress(currentProgress, d_ptr->mTotSteps);
        }

        /* the hash can be cleared now */
        d_ptr->mAttValueHash.clear();

    } /* if(progress != d_ptr->mProgress) */

    qDebug() << __FUNCTION__ << QThread::currentThread() << "Thread finished " << isFinished();
}

void Extractor::cancel()
{
    d_ptr->mCancel = true;
}

bool Extractor::hasError()
{
    return d_ptr->mErrors.size() > 0;
}

void Extractor::clearErrors()
{
    d_ptr->mErrors.clear();
}

void Extractor::threadFinished()
{
    /* the mHdbExtractor is deleted next time the thread is run() or
     * by ~Extractor()
     */
    qDebug() << __FUNCTION__ << "hasError" << hasError();

    if(hasError())
        qApp->postEvent(this, new XErrorEvent(d_ptr->mErrors));

    /* IMPORTANT: progress event with step == total only at the end, after all events!
     * This is emitted when thread finishes, regardless errors
     */
    qApp->postEvent(this, new ProgressEvent(d_ptr->mTotSteps, d_ptr->mTotSteps));
}

AttInfo::Type Extractor::mGetAttInfoType(XVariant::DataFormat f, XVariant::DataType dt, XVariant::Writable w)
{
    AttInfo::Type at;
    if((w == XVariant::RO || w == XVariant::WO) && f == XVariant::Scalar && dt != XVariant::String)
        at = AttInfo::scNum1;
    else if((w == XVariant::RO || w == XVariant::WO) && f == XVariant::Scalar && dt == XVariant::String)
        at = AttInfo::scStr1;
    else if(w == XVariant::RW && f == XVariant::Scalar && dt  != XVariant::String)
        at = AttInfo::scNum2;
    else if(w == XVariant::RW && f == XVariant::Scalar && dt  == XVariant::String)
        at = AttInfo::scStr2;
    else if((w == XVariant::RO || w == XVariant::WO) && f == XVariant::Vector && dt != XVariant::String)
        at = AttInfo::sp1;
    else if(w == XVariant::RW && f == XVariant::Vector && dt != XVariant::String)
        at = AttInfo::sp2;
    else if((w == XVariant::RO || w == XVariant::WO) && f == XVariant::Matrix)
        at = AttInfo::Im1;
    else if(w == XVariant::RW && f == XVariant::Matrix)
        at = AttInfo::Im2;
    return at;
}
