#include "selectfromdiarydialog.h"
#include <QTreeWidget>
#include <QSettings>
#include <QTWriter>
#include <QtDebug>
#include <elettracolors.h>
#include "extraction/extractor.h"

SelectFromDiaryDialog::SelectFromDiaryDialog(QWidget *parent, const QString& diarySource) : QDialog(parent)
{
    ui = new Ui::Diary;
    ui->setupUi(this);
    mDiarySource = diarySource;
    QSettings s;
    QDateTime startDateTime = s.value("DIARY_START_DATE_TIME", QDateTime::currentDateTime()).toDateTime().addMonths(-6);
    QDateTime stopDateTime = QDateTime::currentDateTime();

    Extractor * extractor = new Extractor(this);
    connect(extractor, SIGNAL(progress(int,int)), this, SLOT(extractionProgress(int,int)));
    //  connect(extractor, SIGNAL(progress(int,int)), ui->progressBar, SLOT(setPBValue(int,int)));
    connect(extractor, SIGNAL(onError(QString,QString)),
            this, SLOT(onError(QString,QString)));
    connect(extractor, SIGNAL(attValueListReady(const QList<AttData>& ,AttInfo::Type)),
            this, SLOT(update(const QList<AttData>& , AttInfo::Type)));
    

    ui->dteStart->setDateTime(startDateTime);
    ui->dteStop->setDateTime(stopDateTime);
    ui->sbLast->setValue(s.value("DIARY_LAST_HOURS", 12).toInt()); /* default 12 (months) */
    ui->cbPeriod->setCurrentIndex(s.value("DIARY_CB_LAST_INDEX", 3).toInt()); /* default index 3: months */

    connect(ui->pbLoad, SIGNAL(clicked()), this, SLOT(accept()));
    connect(ui->pbClose, SIGNAL(clicked()), this, SLOT(close()));
    connect(ui->pbRefresh, SIGNAL(clicked()), this, SLOT(load()));
    connect(ui->sbLast, SIGNAL(valueChanged(int)), this, SLOT(setInterval()));
    connect(ui->cbPeriod, SIGNAL(currentIndexChanged(int)), this, SLOT(setInterval()));
    connect(ui->pbNow, SIGNAL(clicked()), this, SLOT(setStopDateNow()));

    connect(ui->treeWidget, SIGNAL(itemSelectionChanged()), this, SLOT(selectionChanged()));
    connect(ui->treeWidget, SIGNAL(itemEntered(QTreeWidgetItem*,int)), this, SLOT(itemEntered(QTreeWidgetItem*,int)));

    ui->pbLoad->setDisabled(true);

    connect(ui->pbSaveOnDiary, SIGNAL(clicked()), this, SLOT(saveOnDiary()));
    connect(ui->teDiary, SIGNAL(textChanged()), this, SLOT(diaryTextChanged()));
    ui->pbSaveOnDiary->setDisabled(true);

    ui->treeWidget->sortByColumn(1, Qt::DescendingOrder);
    load();
}

SelectFromDiaryDialog::~SelectFromDiaryDialog()
{
    printf("\e[1;31mDELETE\e[0m\n");
}

void SelectFromDiaryDialog::showEvent(QShowEvent *e)
{
    QDialog::showEvent(e);
    ui->treeWidget->header()->resizeSection(0, ui->treeWidget->width() * 2.0/3.0);
}

void SelectFromDiaryDialog::closeEvent(QCloseEvent *e)
{
    QSettings s;
    s.setValue("DIARY_START_DATE_TIME", ui->dteStart->dateTime());
    /* don't save stop date time because when panel starts, stop time is now */
    //    s.setValue("DIARY_STOP_DATE_TIME", ui->dteStop->dateTime());
    s.setValue("DIARY_LAST_HOURS", ui->sbLast->value());
    s.setValue("DIARY_CB_LAST_INDEX", ui->cbPeriod->currentIndex());

    QDialog::closeEvent(e);
}

QDateTime SelectFromDiaryDialog::selectedDateTime() const
{
    return mSelectedDateTime;
}

QString SelectFromDiaryDialog::comment() const
{
    return mComment;
}

void SelectFromDiaryDialog::extractionProgress(int step, int total)
{
    ui->progressBar->setPBValue(step, total);
}

QTreeWidgetItem *SelectFromDiaryDialog::findItem(const QDateTime &dt, const QString &s) const
{
    for(int i = 0; i < ui->treeWidget->topLevelItemCount(); i++)
    {
        QTreeWidgetItem *ith = ui->treeWidget->topLevelItem(i);
        //  printf("\e[1;33manalyzing %s\e[0m\n", qstoc(ith->text(0)));
        if(ith->data(1, Qt::UserRole).toDateTime() == dt && ith->data(0, Qt::ToolTipRole).toString() == s)
            return ith;
    }
    return NULL;
}

void SelectFromDiaryDialog::startHighlightingRecent()
{
    QTimer *updateRecentTimer = findChild<QTimer *>("updateRecent");
    if(updateRecentTimer)
        delete updateRecentTimer;

    updateRecentTimer = new QTimer(this);
    updateRecentTimer->setObjectName("updateRecent");
    updateRecentTimer->setSingleShot(false);
    updateRecentTimer->setInterval(600);
    updateRecentTimer->setProperty("timeoutCnt", 0);
    connect(updateRecentTimer, SIGNAL(timeout()), this, SLOT(highlightRecent()));
    updateRecentTimer->start();
    highlightRecent();
}

void SelectFromDiaryDialog::update(const QList<AttData> & ad, AttInfo::Type)
{
    for(int i = ad.size() - 1; i >= 0; i--)
    {
        const AttData& d = ad[i];
        if(d.valid())
        {
            QDateTime dt = d.dateTime;
            QString txt = d.value;
            QTreeWidgetItem *it = findItem(dt, txt);
            if(!it)
            {
                /* create new item */
                txt.chop(txt.length() - 30);
                QStringList columns = QStringList()  << txt << dt.toString("yyyy-MM-dd hh:mm:ss");
                QTreeWidgetItem *it = new QTreeWidgetItem(ui->treeWidget, columns);
                it->setData(0, Qt::ToolTipRole, d.value);
                it->setData(1, Qt::UserRole, dt);
            }
        }
    }
    /* if elements are removed from the database, the application must be restarted
    */

    ui->labelDiary->setText("Diary read on " + QDateTime::currentDateTime().toString("yyyy-MM-dd hh:mm:ss"));
    startHighlightingRecent();
}

void SelectFromDiaryDialog::onError(QString, QString)
{

}

void SelectFromDiaryDialog::setInterval()
{
    QDateTime stopDateTime = QDateTime::currentDateTime();
    ui->dteStop->setDateTime(stopDateTime);
    QString interval = ui->cbPeriod->currentText();
    if(interval == "hours")
        ui->dteStart->setDateTime(stopDateTime.addSecs(-60 * 60 * ui->sbLast->value()));
    else if(interval == "days")
        ui->dteStart->setDateTime(stopDateTime.addDays(-ui->sbLast->value()));
    else if(interval == "weeks")
        ui->dteStart->setDateTime(stopDateTime.addDays(-ui->sbLast->value() * 7));
    else if(interval == "months")
        ui->dteStart->setDateTime(stopDateTime.addMonths(-ui->sbLast->value()));
}

void SelectFromDiaryDialog::setStopDateNow()
{
    ui->dteStop->setDateTime(QDateTime::currentDateTime());
    this->startHighlightingRecent();
}

void SelectFromDiaryDialog::load()
{
    Extractor *extractor = findChild<Extractor *>();
    if(extractor->isRunning())
        extractor->cancel();
    QStringList attrs = QStringList() << mDiarySource;
    extractor->getData(ui->dteStart->dateTime(), ui->dteStop->dateTime(), attrs);
}

void SelectFromDiaryDialog::selectionChanged()
{
    QList<QTreeWidgetItem *> sel = ui->treeWidget->selectedItems();
    ui->pbLoad->setEnabled(sel.size() > 0);
    if(sel.size() > 0)
    {
        mSelectedDateTime = sel.first()->data(1, Qt::UserRole).toDateTime();
        mComment = sel.first()->data(0, Qt::ToolTipRole).toString();
    }
    else
    {
        mComment = QString();
        mSelectedDateTime = QDateTime::currentDateTime();
    }
}

void SelectFromDiaryDialog::itemEntered(QTreeWidgetItem *it, int column)
{
    ui->textEdit->setText(mMakeHtml(it));
}


QString SelectFromDiaryDialog::mMakeHtml(const QTreeWidgetItem* it) const
{
    QString html = "<html>\n";
    html = "<h4>" + it->data(1, Qt::UserRole).toDateTime().toLocalTime().toString() + "</h4>\n";
    html += "<p>" + it->data(0, Qt::ToolTipRole).toString() + "</p>";
    html += "</html>";
    return html;
}

void SelectFromDiaryDialog::setDiarySource(const QString &src)
{
    mDiarySource = src;
}

void SelectFromDiaryDialog::saveOnDiary()
{
    QString txt = ui->teDiary->toPlainText();
    if(!mDiarySource.isEmpty() && !txt.isEmpty())
    {
        QTWriter *w = new QTWriter(this);
        w->setAutoDestroy(true);
        connect(w, SIGNAL(connectionOk(bool)), this, SLOT(diaryDevConnectionOk(bool)));
        w->setTargets(mDiarySource);
        w->execute(txt);
        ui->labelDiary->setText("Reloading diary in 3 seconds...");
        QTimer *timer = new QTimer(this);
        timer->setObjectName("diaryReload");
        timer->setInterval(1000);
        timer->setProperty("timeoutCnt", 0);
        timer->setSingleShot(false);
        connect(timer, SIGNAL(timeout()), this, SLOT(diaryReloadTimeout()));
        timer->start();
    }

}

void SelectFromDiaryDialog::diaryTextChanged()
{
    QRegExp r("\\s*");
    ui->pbSaveOnDiary->setDisabled(r.exactMatch(ui->teDiary->toPlainText()));
}

void SelectFromDiaryDialog::diaryDevConnectionOk(bool ok)
{
    if(!ok)
    {
        ui->labelDiary->setText("Error connecting to diary dev");
        QTWriter *w = qobject_cast<QTWriter *>(sender());
        ui->labelDiary->setToolTip(w->qtangoComHandle()->connectionStatus());
        QTimer *t = findChild<QTimer *>("diaryReload");
        if(t)
            delete t;
    }
    else
        ui->labelDiary->setToolTip("connection ok");
}

void SelectFromDiaryDialog::diaryReloadTimeout()
{
    QTimer *t = qobject_cast<QTimer *>(sender());
    int timeout = t->property("timeoutCnt").toInt();
    t->setProperty("timeoutCnt", ++timeout);
    if(timeout == 3)
    {
        ui->dteStop->setDateTime(QDateTime::currentDateTime());
        load();
        delete t;
    }
    else
        ui->labelDiary->setText(QString("Reloading diary in %1 seconds...").arg(3 - timeout));

}

void SelectFromDiaryDialog::highlightRecent()
{
    int alpha = 255;
    int alphamax = -1;
    QColor g(Qt::white);
    QTimer *updateRecentTimer = findChild<QTimer *>("updateRecent");
    int timeoutcnt = updateRecentTimer->property("timeoutCnt").toInt();
    updateRecentTimer->setProperty("timeoutCnt", ++timeoutcnt);
    foreach(QTreeWidgetItem *it, ui->treeWidget->findItems("*", Qt::MatchWildcard))
    {
        QDateTime dt = it->data(1, Qt::UserRole).toDateTime();
        if(dt.secsTo(QDateTime::currentDateTime()) < 15000)
        {
            /* highlight recently added items */
            g = it->backgroundColor(0);
            if(timeoutcnt == 1)
            {
                g = QColor(Qt::green);
                g.setAlpha(140);
            }
            alpha = g.alpha() - 2;
            if(alpha > 38)
            {
                g.setAlpha(alpha);
                it->setBackground(0, g);
                it->setBackground(1, g);

            }
            else
            {
                it->setBackground(0, QBrush(Qt::NoBrush));
                it->setBackground(1, QBrush(Qt::NoBrush));
            }
            if(alphamax < alpha)
                alphamax = alpha;
        }
    }
    /* stop date time also: */
    if(ui->dteStop->dateTime().secsTo(QDateTime::currentDateTime()) < 60)
    {
        QColor dtcolor;
        if(timeoutcnt == 1)
        {
            dtcolor = QColor(Qt::green);
            dtcolor.setAlpha(140);
        }
        else
            dtcolor = ui->eLedNow->property("color").value<QColor>();

        alpha = dtcolor.alpha() - 2;
        if(alpha > 38)
        {
            dtcolor.setAlpha(alpha);
        }
        else
        {
            dtcolor = QColor(KGRAY);
        }
        ui->eLedNow->setProperty("color", dtcolor);
        ui->eLedNow->setColor(dtcolor);
        ui->eLedNow->setAlphaChannel(dtcolor.alpha());

        if(alphamax < alpha)
            alphamax = alpha;

    }
    //    printf("\e[1;36mhighlightRecent: alpha %d\e[0m\n", alpha);
    if(alphamax <= 38 || alphamax == 0) /* no recent  */
    {
        /* restore background color */
        printf("\e[1;31mdeleting recent timer \e[0m\n");
        delete updateRecentTimer;
    }
}
