#include "timemachine.h"
#include "timemachineinfoviewdbhelper.h"
#include "ui_timemachine.h"
#include "database/snapdb.h"
#include "database/attdata.h"
#include "database/attinfoquery.h"
#include "save/save.h"
#include "restore/snapselectionmanager.h"
#include "led.h"
#include "about.h"
#include "progressbar.h"
#include "uihelper.h"
#include "restore/restore.h"
#include "diff/diffdialog.h"
#include "selection/selectiondialog.h"
#include "selection/selection.h"
#include "settings/settingsdialog.h"
#include "settings/settings.h"
#include "log/logger.h"

#include "extraction/extractor.h"
#include <QMetaType>
#include <QTimer>
#include <action_factory.h>
#include <QMessageBox>
#include <QtDebug>


TimeMachine::TimeMachine(QWidget *parent) :
    QMainWindow(parent),
    ui(new Ui::TimeMachine)
{
    /* act on ActionFactory to disable TLog/TUtil::addLog for all sources targets that
     * will be created by TimeMachine.
     */
#if ((VER_MAJ >3 && VER_MIN > 1 && VER_FIX > 2) || (VER_MAJ >3 && VER_MIN > 2))
    ActionFactory::actionFactory()->setLoggingDisabled(true);
#endif

    ActionFactory::actionFactory()->setDefaultRefreshMode(POLLED_REFRESH);

    ui->setupUi(this);

    ui->snapInfoView->init();
    ui->gbErrors->setHidden(true);

    /* log directory from command line arguments */
    QString logDir = "";
    if(qApp->arguments().contains("--log-dir") && qApp->arguments().count()
            > qApp->arguments().indexOf("--log-dir") + 1)
    {
        logDir = qApp->arguments().at(qApp->arguments().indexOf("--log-dir") + 1);
    }
    Logger *logger = new Logger(logDir);

    /* UI HELPER: manages interface interaction with user, such as
     * shows/hides enables/disables part of the user interface
     * according to determined user actions.
     */
    UiHelper *uiHelper = new UiHelper(this);
    connect(ui->cbExpandContextView, SIGNAL(toggled(bool)), uiHelper, SLOT(expandContextView(bool)));
    connect(uiHelper, SIGNAL(contextViewCollapsed(bool)), ui->cbExpandContextView, SLOT(setChecked(bool)));
    ui->cbExpandContextView->setChecked(false);
    connect(this, SIGNAL(selectionDialogClosed()), uiHelper, SLOT(selectionDialogClosed()));
    connect(this, SIGNAL(selectionDialogShown()), uiHelper, SLOT(selectionDialogShown()));

    /* SNAP DB */
    Settings settings;
    SnapDb *snapDb = new SnapDb(this);
    snapDb->setConnectionParameters(settings.dbConnectionParams());

    connect(snapDb, SIGNAL(contextListReady(const QList<ContextData>&)),
            ui->ctxView, SLOT(update(const QList<ContextData>&)));

    connect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)),
            ui->snapInfoView, SLOT(update(const QList<AttInfo>&)));

    connect(snapDb, SIGNAL(orderedSectionsReady(QStringList)), ui->snapInfoView,
            SLOT(setSectionsList(QStringList)));

    /* as soon as the database is available, obtain the list of contexts and the
     * list of the machine sections, ordered as in the section_order database table.
     */
    connect(snapDb, SIGNAL(databaseOpen()), snapDb, SLOT(getContextList()));
    connect(snapDb, SIGNAL(databaseOpen()), snapDb, SLOT(getOrderedSectionsList()));

    connect(snapDb, SIGNAL(dbConnectionInfo(QString)), statusBar(), SLOT(showMessage(QString)));

    connect(ui->actionReload_data_from_DB, SIGNAL(triggered()), snapDb, SLOT(getContextList()));

    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), snapDb, SLOT(getSnapshotList(int)));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), snapDb, SLOT(getAttInfoList(int)));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), uiHelper, SLOT(contextSelectionChanged(int)));
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), ui->snapInfoView,  SLOT(setContextId(int)));
    connect(ui->snapInfoView, SIGNAL(error(QString,QString)), ui->errorView, SLOT(addError(QString,QString)));

    /* when the user clicks on the context list, he must wait until snap list and snap info
     * are loaded before selecting a snapshot.
     * Moreover, the items in the snapshot list and snap info view must be cleared.
     */
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), ui->snapInfoView, SLOT(clear()));

    /* PROGRESSBAR in status bar */
    ProgressBar * progressBar = new ProgressBar(statusBar());
    statusBar()->addWidget(progressBar, 2);

    /* LED in status bar
     */
    Led *led = new Led(statusBar()); /* would be reparented anyway to statusBar */
    connect(led, SIGNAL(clicked()), uiHelper, SLOT(ensureErrorViewVisible()));
    statusBar()->addPermanentWidget(led);

    /* Errors from database queries */
    connect(snapDb, SIGNAL(querySent()), led, SLOT(setMoving()));
    /* reset led when database is open */
    connect(snapDb, SIGNAL(databaseOpen()), led, SLOT(reset()));
    connect(snapDb, SIGNAL(queryExecuted(bool)), led, SLOT(setOk(bool)));
    connect(snapDb, SIGNAL(databaseError(QString,QString)),
            ui->errorView, SLOT(addError(QString,QString)));
    connect(snapDb, SIGNAL(databaseError(QString,QString)), led, SLOT(setError()));

    /* errors, Application cursor and db related status bar messages */
    connect(snapDb, SIGNAL(queryExecuted(bool)), uiHelper, SLOT(dbDataRetrievalEnd(bool)));
    connect(snapDb, SIGNAL(querySent()), uiHelper, SLOT(dbDataRetrievalStart()));

    /* event though snapDb's query sent SIGNAL should be enough, the two following
     * connections make the led change to moving immediately. Signals coming from
     * snapDb are sent from a different thread. Qt::AutoConnection in THdbBasePrivate
     * connections would queue signal slots.
     */
    connect(ui->ctxView, SIGNAL(contextSelectionChanged(int)), led, SLOT(setMoving()));
    
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), led, SLOT(forceOk()));

    /* ERROR tree view
     */
    connect(ui->errorView, SIGNAL(visible(bool)), ui->gbErrors, SLOT(setVisible(bool)));
    connect(ui->errorView, SIGNAL(visible(bool)), ui->gbErrors, SLOT(setChecked(bool)));
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), ui->errorView, SLOT(clear()));

    /* RESTORE object
     */
    Restore *restore  = new Restore(this, logger);
    connect(ui->pbRestore, SIGNAL(clicked()), this, SLOT(restore()));
    connect(restore, SIGNAL(started()),  led, SLOT(setMoving())); /* LED */
    connect(restore, SIGNAL(started()),  ui->gbErrors, SLOT(hide())); /* HIDE ERRORS on restore click */
    connect(restore, SIGNAL(finished()), this, SLOT(restoreFinished()));
    connect(restore, SIGNAL(progress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(restore, SIGNAL(progress(int,int)), this, SLOT(restorePprogress(int,int)));
    connect(restore, SIGNAL(status(QString)), statusBar(), SLOT(showMessage(const QString&)));
    connect(ui->pbClearErrorHistory, SIGNAL(clicked()), restore, SLOT(clearErrors()));

    /* RESTORE selection manager */
    SnapSelectionManager *snapSelectionManager =
            new SnapSelectionManager(ui->snapInfoView, SnapSelectionManager::SaveByContext);

    printf("\e[1;35m please check if the following commented line needs to be adapted!\e[0m\n");
 //   connect(ui->snapView, SIGNAL(snapSelectionChanged(int)), snapSelectionManager,
 //           SLOT(load(int)));

    connect(ui->snapInfoView, SIGNAL(snapInfoTreeViewSorted(int)), snapSelectionManager,
            SLOT(loadFromId(int)));

    connect(ui->snapInfoView, SIGNAL(clicked(const QModelIndex& )),
            snapSelectionManager, SLOT(itemClicked(const QModelIndex& )));

    /* the following one is necessary after the time machine has changed the way selection is saved.
     * Selection is now saved when item is clicked on the snapInfoView, no more on ItemSelectionChanged.
     * So, it is necessary to trigger saveSelection if we want to programmatically save the selection
     * when not in response to a user's click on the tree view
     */
    connect(ui->pbIKnow, SIGNAL(clicked()), snapSelectionManager, SLOT(saveSelection()));

    /* selection manager error handling. They are not fatal, so it is enough to
     * signal them to the user (simply, the selection is not remembered.
     */
    connect(snapSelectionManager, SIGNAL(error(QString,QString)), ui->errorView,
            SLOT(addError(QString,QString)));

    /* DIFF push button and Diff Dialog */
    connect(ui->pbDiff, SIGNAL(clicked()), this, SLOT(showDiffDialog()));

    /* SELECTION DIALOG */
    connect(ui->snapInfoView, SIGNAL(selectionRequest()), this, SLOT(executeSelectionDialog()));
    /* FILTER view */
    connect(ui->snapInfoView, SIGNAL(showFilterRequest()), uiHelper, SLOT(showSnapViewFilter()));

    /* DATABASE configuration dialog */
    connect(ui->actionDbConnection, SIGNAL(triggered()), this, SLOT(executeConfigDialog()));

    statusBar()->showMessage("Save Restore ready. Click on a context to begin.");

    /* FILTER on snapinfotreeview */
    connect(ui->pbHideFilter, SIGNAL(clicked()), uiHelper, SLOT(hideSnapViewFilter()));
    connect(ui->pbHideFilter, SIGNAL(clicked()), ui->leFilter, SLOT(clear()));
    connect(ui->pbClearFilter, SIGNAL(clicked()), ui->leFilter, SLOT(clear()));
    connect(ui->leFilter, SIGNAL(textChanged(QString)), ui->snapInfoView, SLOT(filter(QString)));
    connect(ui->cbFilterOptions, SIGNAL(currentIndexChanged(int)), ui->snapInfoView, SLOT(setFilterMode(int)));
    connect(ui->leFilter, SIGNAL(textChanged(QString)), uiHelper, SLOT(snapInfoViewFilterTextChanged(QString)));
    /* invalid regular expression */
    connect(ui->snapInfoView, SIGNAL(filterRegexpValid(bool)), uiHelper, SLOT(setFilterLineEditValid(bool)));

    /* ABOUT dialogs */
    About *about = new About(this);
    connect(ui->actionAbout, SIGNAL(triggered()), about, SLOT(about()));
    connect(ui->actionAbout_THdb, SIGNAL(triggered()), about, SLOT(aboutTHdb()));
    connect(ui->actionAbout_QTango, SIGNAL(triggered()), about, SLOT(aboutQTango()));
    connect(ui->actionAbout_Qt, SIGNAL(triggered()), about, SLOT(aboutQt()));
    connect(ui->actionManual, SIGNAL(triggered()), about, SLOT(manual()));

    /* NULL and invalid items management */
    connect(ui->snapInfoView, SIGNAL(invalidItems(int,int)), uiHelper, SLOT(invalidItemsInSnap(int,int)));
    connect(ui->snapInfoView, SIGNAL(allItemsValid(bool)), uiHelper, SLOT(allItemsValidInSnap(bool)));
    connect(ui->pbIKnow, SIGNAL(clicked()), uiHelper, SLOT(iKnowClicked()));
    connect(ui->pbIKnow, SIGNAL(clicked()), snapSelectionManager, SLOT(deselectReadOnlyAndNullItems()));

    Extractor * extractor = new Extractor(this);
    connect(extractor, SIGNAL(progress(int,int)), this, SLOT(extractionProgress(int,int)));
    connect(extractor, SIGNAL(progress(int,int)), progressBar, SLOT(setPBValue(int,int)));
    connect(extractor, SIGNAL(onError(QString,QString)),
            ui->errorView, SLOT(addError(QString,QString)));
    bool res = connect(extractor, SIGNAL(attValueListReady(const QList<AttData>& ,AttInfo::Type)),
            ui->snapInfoView, SLOT(update(const QList<AttData>& , AttInfo::Type)));

    qDebug() << __FUNCTION__ << QThread::currentThread();
    printf("\e[1;32m connection to snap slot %d\e[0m\n\n", res);
    connect(ui->pushFetch, SIGNAL(clicked()), this, SLOT(extractFromHdb()));
    ui->dte->setDateTime(QDateTime::currentDateTime());
}

void TimeMachine::extractFromHdb()
{
    ui->snapInfoView->prepareForAttributeValuesQuery();
    Extractor *extractor = findChild<Extractor *>();
    QDateTime dt = ui->dte->dateTime();
    QStringList attrs = ui->snapInfoView->getAttributeList();
    extractor->getData(dt, attrs);
}

void TimeMachine::restore()
{
    findChild<Restore *>()->restore(ui->snapInfoView);
}

void TimeMachine::restoreProgress(int s, int t)
{
    printf("\e[0;33m progress in restore %d/%d\n", s, t);
}

void TimeMachine::extractionProgress(int s, int t)
{
    printf("\e[0;33m progress in TimeMachine %d/%d\n", s, t);
    if(s == t ) /* finished */
    {
        printf("\e[1;32mTimeMachine::progress: FINISHED, triggering selection on snapInfoView! ctx %d\e[0m\n",
               ui->snapInfoView->currentContextId());
        findChild<SnapSelectionManager *>()->loadFromId(ui->snapInfoView->currentContextId());
    }
}

void TimeMachine::restoreFinished()
{
    Restore *restore = findChild<Restore *>();
    if(restore->writeErrorCount() > 0)
    {
       QHash<QString, QString> tangoErrorHash = restore->tangoErrors();
       foreach(QString key, tangoErrorHash.keys())
          ui->errorView->addError(key, tangoErrorHash.value(key));
       statusBar()->findChild<Led *>()->setError();
    }
    else
        statusBar()->findChild<Led *>()->setOk();
}

void TimeMachine::executeSelectionDialog()
{
    SelectionDialog *selectionDialog = new SelectionDialog(this, ui->ctxView->selectedContextId());
    connect(selectionDialog, SIGNAL(selectionChanged(const QStringList&)),
            ui->snapInfoView, SLOT(selectByDevice(const QStringList &)));

    connect(selectionDialog, SIGNAL(finished(int)), this, SLOT(selectionDialogClosed(int)));

    Selection * selection = new Selection(this, findChild<SnapDb *>(),
              ui->ctxView->selectedContextId(),
              ui->snapInfoView->selectionModel()->selection());

    connect(selection, SIGNAL(typesReady(QStringList)), selectionDialog,
            SLOT(addTypes(QStringList)));
    connect(selection, SIGNAL(deviceReady(QString,QString,QString)),
            selectionDialog, SLOT(addDevice(QString,QString,QString)));
    connect(selection, SIGNAL(dbDataRetrieved()), selectionDialog,
            SLOT(dataLoadedFromDb()));

    /* wait some time to avoid interface freezes */
    QTimer::singleShot(200, selection, SLOT(getSelection()));

    selectionDialog->show();

    emit selectionDialogShown();
}

void TimeMachine::selectionDialogClosed(int result)
{
    Selection * selection = findChild<Selection *>();
    if(result == QDialog::Rejected)
    {
        /* restore original selection, before dialog execution */
        ui->snapInfoView->selectionModel()->clear();
        ui->snapInfoView->selectionModel()->select(selection->savedItemSelection(), QItemSelectionModel::Select);
    }
    delete selection;
    emit selectionDialogClosed();
}

void TimeMachine::executeConfigDialog()
{
    SettingsDialog settingsDialog(this, findChild<SnapDb *>());
    settingsDialog.exec();
}

void TimeMachine::showDiffDialog()
{
    printf("\e[1;31mTimeMachine::showDiffDialog(): NOT IMPLEMENTED\e[0m\n");
//    DiffDialog dlg(this, ui->snapView->selectedSnapIds(), ui->ctxView->selectedContextId());
//    SnapDb *snapDb = findChild<SnapDb *>();
//    dlg.setSnapDb(snapDb);
//    disconnect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)),
//            ui->snapInfoView, SLOT(update(const QList<AttInfo>&)));
//    dlg.getDiff();
//    connect(&dlg, SIGNAL(selectSnapshot(int)), ui->snapView, SLOT(select(int)));
//    dlg.exec();
//    connect(snapDb, SIGNAL(attInfoListReady(const QList<AttInfo>&)),
//      ui->snapInfoView, SLOT(update(const QList<AttInfo>&)));
}

TimeMachine::~TimeMachine()
{
    delete findChild<Extractor *>();
    delete findChild<Selection *>();
    delete findChild<Restore *>();
    delete ui;
}
