#include "timemachineattributedatatreeview.h"
#include "modelview/itemmodel.h"
#include "utils/helperapplication.h"
#include "selection/selectiondialog.h"
#include "settings/settings.h"
#include <QHeaderView>
#include <QVariant>
#include <QContextMenuEvent>
#include <QMenu>
#include <QPainter>
#include <QPaintEvent>
#include <QtDebug>
#include <tutil.h>
#include <macros.h>
#include <elettracolors.h>
#include "treeviewfilter.h"
#include "invaliditemmanager.h"
#include "snapinfotreeviewsorter.h"
#include "uihelper.h"

TimeMachineAttributeDataTreeView::TimeMachineAttributeDataTreeView(QWidget *parent)
    : AttributeDataTreeView(parent)
{

}

void TimeMachineAttributeDataTreeView::init()
{
    setRootIsDecorated(false);

#if QT_VERSION >= 0x050000
    header()->setSectionResizeMode(QHeaderView::ResizeToContents);
#else
    header()->setResizeMode(QHeaderView::Interactive);
#endif
    header()->setToolTip("Click to switch sorting mode");
    setSortingEnabled(true);
    connect(header(), SIGNAL(sectionClicked(int)), this, SLOT(switchSortMode(int)));

    ItemModel *itemModel = new ItemModel(this, getHeaderData());

    setModel(itemModel);

    stateChanged(Init); /* takes care of initializing SelectionMode to NoSelection */

    setSelectionBehavior(QAbstractItemView::SelectRows);
    setDragEnabled(false);
    setDragDropMode(QAbstractItemView::NoDragDrop);

    setStatusTip("Snap Info View: click to select/deselect for restore. Right click "
                 "for context menu.");
    /* selected items green, not blue as the default, indicating that they will be restored */
    QPalette p = palette();
    p.setColor(QPalette::Highlight, KGREEN.lighter(120));
    p.setColor(QPalette::HighlightedText, Qt::black);
    setPalette(p);

    m_invalidItemManager = new InvalidItemManager(this);
    m_currentLastExpectedType = AttInfo::Invalid;

    connect(new TreeViewFilter(this), SIGNAL(regexpValid(bool)), this, SIGNAL(filterRegexpValid(bool)));
}

QList<QVariant> TimeMachineAttributeDataTreeView::getHeaderData()
{
    return QList<QVariant>() << "Attribute" << "Read" << "Time" << " Set Point" << "Type" << "Format" << "Writable";
}

AttributeDataTreeView::Mode TimeMachineAttributeDataTreeView::getMode() const
{
   return AttributeDataTreeView::Hdb;
}

int TimeMachineAttributeDataTreeView::getRestoreColumnIndex() const
{
    return getColumnIndex(ColumnRead);
}

int TimeMachineAttributeDataTreeView::getColumnIndex(Columns c) const
{
    switch(c)
    {
    case ColumnAttr:
        return 0;
    case ColumnRead:
        return 1;
    case ColumnDateTime:
        return 2;
    case ColumnSetPoint:
        return 3;
    case ColumnType:
        return 4;
    case ColumnFormat:
       return 5;
    case ColumnWritable:
        return 6;
    }
    perr("TimeMachineInfoTreeView::getColumnIndex: invalid column %d", c);
    return 0;
}

void TimeMachineAttributeDataTreeView::update(const QList<AttData>& values,  AttInfo::Type type)
{
    printf("\e[1;31mupdate with values size %d type %d\e[0m\n\n\n", values.size(), type);
    ItemModel *itemModel = static_cast<ItemModel *>(model());

    itemModel->blockSignals(true);
    selectionModel()->blockSignals(true);

    foreach(AttData av, values)
    {
        qDebug() << __FUNCTION__ << "adding " << av.name << av.value;
        int row = findRow(av.name);
        if(row < 0)
            perr("TimeMachineInfoTreeView::update: no row with attribute name \"%s\"", qstoc(av.name));
        QString truncValue = av.value;
        QString truncSetPoint = av.setpoint;
        QString dateTime = av.dateTime.toString("yy.MM.dd hh:mm:ss");
        truncValue.truncate(30);
        truncSetPoint.truncate(30);
        /* show the truncated version into the Display Role for performance issues */
        itemModel->setData(model()->index(row, getColumnIndex(ColumnRead)), truncValue, Qt::DisplayRole, true);

        itemModel->setData(model()->index(row, getColumnIndex(ColumnDateTime)),
                               dateTime, Qt::DisplayRole, true);

        /*
         *  IMPORTANT NOTE: save the full data into the user role.
         */
        itemModel->setData(model()->index(row,  getColumnIndex(ColumnRead)), av.value, Qt::UserRole, true);

        itemModel->setData(model()->index(row, getColumnIndex(ColumnRead)), "<p>" + av.value + "</p>",
                           Qt::ToolTipRole, true);

        if(av.type == AttInfo::Im2 || av.type == AttInfo::scNum2 || av.type == AttInfo::sp2
                || av.type == AttInfo::scStr2)
        {
            itemModel->setData(model()->index(row, getColumnIndex(ColumnSetPoint)),
                               truncSetPoint, Qt::DisplayRole, true);
            printf("\e[1;31m setting set point to %s\e[0m\n", qstoc(truncSetPoint));


            /*
             *  IMPORTANT NOTE: save the full data into the user role.
             */
            itemModel->setData(model()->index(row, getColumnIndex(ColumnSetPoint)), av.setpoint, Qt::UserRole, true);

            itemModel->setData(model()->index(row, getColumnIndex(ColumnSetPoint)),  "<p>" + av.setpoint + "</p>",
                               Qt::ToolTipRole, true);
        }
    }

    itemModel->blockSignals(false);
    selectionModel()->blockSignals(false);

    stateChanged(ValuesLoaded);
    /* invalid or read only items in the view ? */
   // if(type == m_currentLastExpectedType)
    {
        bool invalidIts = m_invalidItemManager->findInvalidItems();
        if(invalidIts)
            emit invalidItems(m_invalidItemManager->invalidWriteModeItems(), m_invalidItemManager->nullItems());
        emit allItemsValid(!invalidIts);
        emit attributeValuesDisplayed();
    }
}


void TimeMachineAttributeDataTreeView::prepareForAttributeValuesQuery()
{
    ItemModel *itemModel = static_cast<ItemModel *>(model());
    itemModel->blockSignals(true);
    selectionModel()->blockSignals(true);
    for(int i = 0; i < itemModel->rowCount(QModelIndex()); i++)
    {
        printf("prepare %d\n", i);
        int wri = itemModel->data(itemModel->index(i, (ColumnWritable)), Qt::UserRole).toInt();
        /* prepare Set Point and Read columns with NULL values */
        itemModel->setData(model()->index(i, getColumnIndex(ColumnRead)), "NULL", Qt::DisplayRole, true);
        itemModel->setData(model()->index(i, getColumnIndex(ColumnRead)), "NULL", Qt::ToolTipRole, true);
        itemModel->setData(model()->index(i, getColumnIndex(ColumnDateTime)), "NULL", Qt::DisplayRole, true);
        itemModel->setData(model()->index(i, getColumnIndex(ColumnDateTime)), "NULL", Qt::ToolTipRole, true);

        Tango::AttrWriteType writable = static_cast<Tango::AttrWriteType>(wri);
        if(writable == Tango::READ_WRITE || writable ==  Tango::READ_WITH_WRITE)
        {
            itemModel->setData(model()->index(i, getColumnIndex(ColumnSetPoint)), "NULL", Qt::DisplayRole, true);
            itemModel->setData(model()->index(i, getColumnIndex(ColumnSetPoint)), "NULL", Qt::ToolTipRole, true);
        }
        else
        {
            itemModel->setData(model()->index(i, getColumnIndex(ColumnSetPoint)), "-", Qt::DisplayRole, true);
            itemModel->setData(model()->index(i, getColumnIndex(ColumnSetPoint)), "-", Qt::ToolTipRole, true);
        }
    }
    itemModel->blockSignals(false);
    selectionModel()->blockSignals(false);
}

int TimeMachineAttributeDataTreeView::findRow(const QString& attName)
{
    for(int i = 0; i < model()->rowCount(QModelIndex()); i++)
    {
        if(model()->data(model()->index(i, 0)).toString() == attName)
            return i;
    }
    return -1;
}

